@@ -1,6 +1,6 @@
 #include <net-snmp/net-snmp-config.h>
 
-#ifdef CAN_USE_NLIST
+#ifdef NETSNMP_CAN_USE_NLIST
 #if HAVE_STRING_H
 #include <string.h>
 #else
@@ -62,14 +62,14 @@ auto_nlist_value(const char *string)
          * allocate an extra byte for inclusion of a preceding '_' later 
          */
         it->nl[0].n_name = (char *) malloc(strlen(string) + 2);
-#ifdef aix4
+#if defined(aix4) || defined(aix5)
         strcpy(it->nl[0].n_name, string);
 #else
         sprintf(it->nl[0].n_name, "_%s", string);
 #endif
         it->nl[1].n_name = 0;
         init_nlist(it->nl);
-#ifndef aix4
+#if !(defined(aix4) || defined(aix5))
         if (it->nl[0].n_type == 0) {
             strcpy(it->nl[0].n_name, string);
             init_nlist(it->nl);
@@ -83,7 +83,7 @@ auto_nlist_value(const char *string)
 	    }
             return (-1);
         } else {
-            DEBUGMSGTL(("auto_nlist", "nlist:  found symbol %s at %x.\n",
+            DEBUGMSGTL(("auto_nlist:auto_nlist_value", "found symbol %s at %x.\n",
                         it->symbol, it->nl[0].n_value));
             return (it->nl[0].n_value);
         }
@@ -114,7 +114,7 @@ auto_nlist(const char *string, char *var, int size)
 static void
 init_nlist(struct nlist nl[])
 {
-#ifdef CAN_USE_NLIST
+#ifdef NETSNMP_CAN_USE_NLIST
     int             ret;
 #if HAVE_KVM_OPENFILES
     kvm_t          *kernel;
@@ -142,9 +142,9 @@ init_nlist(struct nlist nl[])
     }
     kvm_close(kernel);
 #else                           /* ! HAVE_KVM_OPENFILES */
-#if defined(aix4) && defined(HAVE_KNLIST)
+#if (defined(aix4) || defined(aix5)) && defined(HAVE_KNLIST)
     if (knlist(nl, 1, sizeof(struct nlist)) == -1) {
-        DEBUGMSGTL(("auto_nlist", "knlist failed on symbol:  %s\n",
+        DEBUGMSGTL(("auto_nlist:init_nlist", "knlist failed on symbol:  %s\n",
                     nl[0].n_name));
         if (errno == EFAULT) {
             nl[0].n_type = 0;
@@ -172,22 +172,22 @@ init_nlist(struct nlist nl[])
 #endif                          /*aix4 */
 #endif                          /* ! HAVE_KVM_OPENFILES */
     for (ret = 0; nl[ret].n_name != NULL; ret++) {
-#ifdef aix4
+#if defined(aix4) || defined(aix5)
         if (nl[ret].n_type == 0 && nl[ret].n_value != 0)
             nl[ret].n_type = 1;
 #endif
         if (nl[ret].n_type == 0) {
             if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, 
 					NETSNMP_DS_AGENT_NO_ROOT_ACCESS)) {
-                DEBUGMSGTL(("auto_nlist", "nlist err:  %s not found\n",
+                DEBUGMSGTL(("auto_nlist:init_nlist", "nlist err:  %s not found\n",
                             nl[ret].n_name));
 	    }
         } else {
-            DEBUGMSGTL(("auto_nlist", "nlist: %s 0x%X\n", nl[ret].n_name,
+            DEBUGMSGTL(("auto_nlist:init_nlist", "nlist: %s 0x%X\n", nl[ret].n_name,
                         (unsigned int) nl[ret].n_value));
         }
     }
-#endif                          /* CAN_USE_NLIST */
+#endif                          /* NETSNMP_CAN_USE_NLIST */
 }
 
 int
@@ -228,11 +228,11 @@ auto_nlist_print_tree(int indent, struct autonlist *ptr)
     }
 }
 #endif
-#else                           /* !CAN_USE_NLIST */
+#else                           /* !NETSNMP_CAN_USE_NLIST */
 #include <net-snmp/agent/auto_nlist.h>
 int
 auto_nlist_noop(void)
 {
     return 0;
 }
-#endif                          /* CAN_USE_NLIST */
+#endif                          /* NETSNMP_CAN_USE_NLIST */
