@@ -14,11 +14,6 @@
 #include <net-snmp/agent/instance.h>
 #include <net-snmp/agent/scalar.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
-
 /** @defgroup watcher watcher
  *  Watch a specified variable and process it as an instance or scalar object
  *  @ingroup leaf
@@ -197,6 +192,16 @@ netsnmp_get_watched_timestamp_handler(void)
     return ret;
 }
 
+int
+netsnmp_watched_timestamp_register(netsnmp_mib_handler *whandler,
+                                   netsnmp_handler_registration *reginfo,
+                                   marker_t timestamp)
+{
+    whandler->myvoid = (void *)timestamp;
+    netsnmp_inject_handler(reginfo, whandler);
+    return netsnmp_register_scalar(reginfo);   /* XXX - or instance? */
+}
+
 int
 netsnmp_register_watched_timestamp(netsnmp_handler_registration *reginfo,
                                    marker_t timestamp)
@@ -204,9 +209,8 @@ netsnmp_register_watched_timestamp(netsnmp_handler_registration *reginfo,
     netsnmp_mib_handler *whandler;
 
     whandler         = netsnmp_get_watched_timestamp_handler();
-    whandler->myvoid = (void *)timestamp;
-    netsnmp_inject_handler(reginfo, whandler);
-    return netsnmp_register_scalar(reginfo);   /* XXX - or instance? */
+
+    return netsnmp_watched_timestamp_register(whandler, reginfo, timestamp);
 }
 
 
@@ -238,7 +242,10 @@ netsnmp_watched_timestamp_handler(netsnmp_mib_handler *handler,
          * data requests 
          */
     case MODE_GET:
-        uptime = netsnmp_marker_uptime( timestamp );
+        if (handler->flags & NETSNMP_WATCHER_DIRECT)
+            uptime = * (long*)timestamp;
+        else
+            uptime = netsnmp_marker_uptime( timestamp );
         snmp_set_var_typed_value(requests->requestvb,
                                  ASN_TIMETICKS,
                                  (u_char *) &uptime,
@@ -293,7 +300,6 @@ netsnmp_register_watched_spinlock(netsnmp_handler_registration *reginfo,
     netsnmp_inject_handler(reginfo, whandler);
     return netsnmp_register_watched_scalar(reginfo, winfo);
 }
-/**  @} */
 
 
 int
@@ -344,3 +350,126 @@ netsnmp_watched_spinlock_handler(netsnmp_mib_handler *handler,
     /* next handler called automatically - 'AUTO_NEXT' */
     return SNMP_ERR_NOERROR;
 }
+
+    /***************************
+     *
+     *   Convenience registration routines - modelled on
+     *   the equivalent netsnmp_register_*_instance() calls
+     *
+     ***************************/
+
+int
+netsnmp_register_ulong_scalar(const char *name,
+                              oid * reg_oid, size_t reg_oid_len,
+                              u_long * it,
+                              Netsnmp_Node_Handler * subhandler)
+{
+    return netsnmp_register_watched_scalar(
+               netsnmp_create_handler_registration(
+                   name, subhandler,
+                   reg_oid, reg_oid_len,
+                   HANDLER_CAN_RWRITE ),
+               netsnmp_create_watcher_info(
+                   (void *)it, sizeof( u_long ),
+                   ASN_UNSIGNED, WATCHER_FIXED_SIZE ));
+}
+
+int
+netsnmp_register_read_only_ulong_scalar(const char *name,
+                              oid * reg_oid, size_t reg_oid_len,
+                              u_long * it,
+                              Netsnmp_Node_Handler * subhandler)
+{
+    return netsnmp_register_watched_scalar(
+               netsnmp_create_handler_registration(
+                   name, subhandler,
+                   reg_oid, reg_oid_len,
+                   HANDLER_CAN_RONLY ),
+               netsnmp_create_watcher_info(
+                   (void *)it, sizeof( u_long ),
+                   ASN_UNSIGNED, WATCHER_FIXED_SIZE ));
+}
+
+int
+netsnmp_register_long_scalar(const char *name,
+                              oid * reg_oid, size_t reg_oid_len,
+                              long * it,
+                              Netsnmp_Node_Handler * subhandler)
+{
+    return netsnmp_register_watched_scalar(
+               netsnmp_create_handler_registration(
+                   name, subhandler,
+                   reg_oid, reg_oid_len,
+                   HANDLER_CAN_RWRITE ),
+               netsnmp_create_watcher_info(
+                   (void *)it, sizeof( long ),
+                   ASN_INTEGER, WATCHER_FIXED_SIZE ));
+}
+
+int
+netsnmp_register_read_only_long_scalar(const char *name,
+                              oid * reg_oid, size_t reg_oid_len,
+                              long * it,
+                              Netsnmp_Node_Handler * subhandler)
+{
+    return netsnmp_register_watched_scalar(
+               netsnmp_create_handler_registration(
+                   name, subhandler,
+                   reg_oid, reg_oid_len,
+                   HANDLER_CAN_RONLY ),
+               netsnmp_create_watcher_info(
+                   (void *)it, sizeof( long ),
+                   ASN_INTEGER, WATCHER_FIXED_SIZE ));
+}
+
+
+int
+netsnmp_register_int_scalar(const char *name,
+                              oid * reg_oid, size_t reg_oid_len,
+                              int * it,
+                              Netsnmp_Node_Handler * subhandler)
+{
+    return netsnmp_register_watched_scalar(
+               netsnmp_create_handler_registration(
+                   name, subhandler,
+                   reg_oid, reg_oid_len,
+                   HANDLER_CAN_RWRITE ),
+               netsnmp_create_watcher_info(
+                   (void *)it, sizeof( int ),
+                   ASN_INTEGER, WATCHER_FIXED_SIZE ));
+}
+
+int
+netsnmp_register_read_only_int_scalar(const char *name,
+                              oid * reg_oid, size_t reg_oid_len,
+                              int * it,
+                              Netsnmp_Node_Handler * subhandler)
+{
+    return netsnmp_register_watched_scalar(
+               netsnmp_create_handler_registration(
+                   name, subhandler,
+                   reg_oid, reg_oid_len,
+                   HANDLER_CAN_RONLY ),
+               netsnmp_create_watcher_info(
+                   (void *)it, sizeof( int ),
+                   ASN_INTEGER, WATCHER_FIXED_SIZE ));
+}
+
+
+int
+netsnmp_register_read_only_counter32_scalar(const char *name,
+                              oid * reg_oid, size_t reg_oid_len,
+                              u_long * it,
+                              Netsnmp_Node_Handler * subhandler)
+{
+    return netsnmp_register_watched_scalar(
+               netsnmp_create_handler_registration(
+                   name, subhandler,
+                   reg_oid, reg_oid_len,
+                   HANDLER_CAN_RONLY ),
+               netsnmp_create_watcher_info(
+                   (void *)it, sizeof( u_long ),
+                   ASN_COUNTER, WATCHER_FIXED_SIZE ));
+}
+/**  @} */
+
