@@ -25,6 +25,7 @@
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include "snmpd.h"
 #include "agentx/agentx_config.h"
+#include "agentx/protocol.h"
 
 /* ---------------------------------------------------------------------
  *
@@ -181,15 +182,13 @@ agentx_register_config_handler(const char *token,
 {
     DEBUGMSGTL(("agentx_register_app_config_handler",
                 "registering .conf token for \"%s\"\n", token));
-    register_app_config_handler(token, parser, releaser, help);
-    register_config_handler("agentx", token, parser, releaser, help);
+    register_config_handler(":agentx", token, parser, releaser, help);
 }
 
 void
 agentx_unregister_config_handler(const char *token)
 {
-    unregister_app_config_handler(token);
-    unregister_config_handler("agentx", token);
+    unregister_config_handler(":agentx", token);
 }
 
 void
@@ -201,6 +200,11 @@ agentx_config_init(void)
     /*
      * Common tokens for master/subagent
      */
+    netsnmp_register_default_domain("agentx", "unix");
+    netsnmp_register_default_target("agentx", "unix", NETSNMP_AGENTX_SOCKET);
+#define val(x) __STRING(x)
+    netsnmp_register_default_target("agentx", "tcp", ":" val(AGENTX_PORT));
+#undef val
     agentx_register_config_handler("agentxsocket",
                                   agentx_parse_agentx_socket, NULL,
                                   "AgentX bind address");
