@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: interface_ioctl.c,v 1.12.2.1 2006/01/25 16:26:38 dts12 Exp $
+ * $Id: interface_ioctl.c 15638 2006-12-06 14:34:11Z rstory $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -259,6 +259,19 @@ netsnmp_access_interface_ioctl_flags_get(int fd,
             ifentry->admin_status = IFADMINSTATUS_DOWN;
             ifentry->oper_status = IFOPERSTATUS_DOWN;
         }
+
+        /*
+         * ifConnectorPresent description:
+         *   This object has the value 'true(1)' if the interface sublayer has a
+         *   physical connector and the value 'false(2)' otherwise."
+         * So, at very least, false(2) should be returned for loopback devices.
+         */
+        if(ifentry->os_flags & IFF_LOOPBACK) {
+            ifentry->connector_present = 0;
+        }
+        else {	
+            ifentry->connector_present = 1;
+        }
     }
     
     return rc;
@@ -386,11 +399,14 @@ netsnmp_access_interface_ioctl_ifindex_get(int fd, const char *name)
     struct ifreq    ifrq;
     int rc = 0;
 
-    DEBUGMSGTL(("access:interface:ioctl", "mtu_get\n"));
+    DEBUGMSGTL(("access:interface:ioctl", "ifindex_get\n"));
 
     rc = _ioctl_get(fd, SIOCGIFINDEX, &ifrq, name);
-    if (rc < 0)
+    if (rc < 0) {
+        DEBUGMSGTL(("access:interface:ioctl",
+                   "ifindex_get error on inerface '%s'\n", name));
         return 0;
+    }
 
     return ifrq.ifr_ifindex;
 #endif /* SIOCGIFINDEX */
