@@ -1,10 +1,20 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: interface_linux.c,v 1.20.2.8 2006/04/25 17:36:04 hardaker Exp $
+ * $Id: interface_linux.c 16595 2007-07-06 21:14:03Z rstory $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
+
+#ifdef HAVE_LINUX_ETHTOOL_H
+#include <linux/types.h>
+typedef __u64 u64;         /* hack, so we may include kernel's ethtool.h */
+typedef __u32 u32;         /* ditto */
+typedef __u16 u16;         /* ditto */
+typedef __u8 u8;           /* ditto */
+#include <linux/ethtool.h>
+#endif /* HAVE_LINUX_ETHTOOL_H */
+
 #include "mibII/mibII_common.h"
 #include "if-mib/ifTable/ifTable_constants.h"
 
@@ -17,34 +27,69 @@
 #endif
 
 #include <net-snmp/data_access/interface.h>
+#include <net-snmp/data_access/ipaddress.h>
 #include "if-mib/data_access/interface.h"
 #include "interface_ioctl.h"
 
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
-#ifdef HAVE_LINUX_ETHTOOL_H
-#include <sys/types.h>
-typedef unsigned long long u64;         /* hack, so we may include kernel's ethtool.h */
-typedef __uint32_t u32;         /* ditto */
-typedef __uint16_t u16;         /* ditto */
-typedef __uint8_t u8;           /* ditto */
-#include <linux/ethtool.h>
-#endif /* HAVE_LINUX_ETHTOOL_H */
 #include <linux/sockios.h>
 
-unsigned int
-netsnmp_arch_interface_get_if_speed(int fd, const char *name);
-#ifdef HAVE_LINUX_ETHTOOL_H
-unsigned int
-netsnmp_arch_interface_get_if_speed_mii(int fd, const char *name);
+#ifndef IF_NAMESIZE
+#define IF_NAMESIZE 16
 #endif
 
+unsigned int
+netsnmp_linux_interface_get_if_speed(int fd, const char *name);
+#ifdef HAVE_LINUX_ETHTOOL_H
+unsigned int
+netsnmp_linux_interface_get_if_speed_mii(int fd, const char *name);
+#endif
+
+#define PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS "/proc/sys/net/ipv%d/neigh/%s/retrans_time_ms"
+#define PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME    "/proc/sys/net/ipv%d/neigh/%s/retrans_time"
+static const char *proc_sys_retrans_time;
+static unsigned short retrans_time_factor = 1;
+
+
+#define PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME_MS "/proc/sys/net/ipv%d/neigh/%s/base_reachable_time_ms"
+#define PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME "/proc/sys/net/ipv%d/neigh/%s/base_reachable_time"
+static const char *proc_sys_basereachable_time;
+static unsigned short basereachable_time_ms = 0;
 void
 netsnmp_arch_interface_init(void)
 {
     /*
-     * nothing to do
+     * Check which retransmit time interface is available
      */
+    char proc_path[ 64+IF_NAMESIZE];
+    char proc_path2[64+IF_NAMESIZE];
+    struct stat st;
+
+    snprintf(proc_path,  sizeof(proc_path),
+             PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS, 6, "default");
+    snprintf(proc_path2, sizeof(proc_path2),
+             PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS, 4, "default");
+
+    if ((stat(proc_path, &st) == 0) || (stat(proc_path2, &st) == 0)) {
+        proc_sys_retrans_time = PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS;
+    } else {
+        proc_sys_retrans_time = PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME;
+        retrans_time_factor = 10;
+    }
+
+    snprintf(proc_path,  sizeof(proc_path),  PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME_MS, 6, "default");
+    snprintf(proc_path2,  sizeof(proc_path),  PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME, 4, "default");
+
+    if ((stat(proc_path, &st) == 0) || (stat(proc_path2, &st) == 0)) {
+        proc_sys_basereachable_time = PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME_MS;
+        basereachable_time_ms = 1;
+    }
+    else {
+        proc_sys_basereachable_time = PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME;
+    }
 }
 
 /*
@@ -64,18 +109,190 @@ netsnmp_arch_interface_index_find(const char *name)
 
 
 /*
- *
- * @retval  0 success
- * @retval -1 no container specified
- * @retval -2 could not open /proc/net/dev
- * @retval -3 could not create entry (probably malloc)
+ * check for ipv6 addresses
  */
-int
-netsnmp_arch_interface_container_load(netsnmp_container* container,
-                                      u_int load_flags)
+void
+_arch_interface_has_ipv6(oid if_index, u_int *flags,
+                         netsnmp_container *addr_container)
 {
-    FILE           *devin;
+#ifdef NETSNMP_ENABLE_IPV6
+    netsnmp_ipaddress_entry *addr_entry = NULL;
+    netsnmp_iterator        *addr_it = NULL;
+    u_int                    addr_container_flags = 0; /* must init to 0 */
+#endif
+
+    if (NULL == flags)
+        return;
+
+    *flags &= ~NETSNMP_INTERFACE_FLAGS_HAS_IPV6;
+
+#ifdef NETSNMP_ENABLE_IPV6
+    /*
+     * get ipv6 addresses
+     */
+    if (NULL == addr_container) {
+        /*
+         * we only care about ipv6, if we need to allocate our own
+         * temporary container. set the flags (which we also use later
+         * to determine if we need to free the container).
+         */
+        addr_container_flags = NETSNMP_ACCESS_IPADDRESS_LOAD_IPV6_ONLY;
+        addr_container =
+            netsnmp_access_ipaddress_container_load(NULL,
+                                                    addr_container_flags);
+        if (NULL == addr_container) {
+            DEBUGMSGTL(("access:ifcontainer",
+                        "couldn't get ip addresses container\n"));
+            return;
+        }
+    }
+    else {
+        /*
+         * addr_container flags must be 0, so we don't release the
+         * user's container.
+         */
+        netsnmp_assert(0 == addr_container_flags);
+    }
+
+
+    /*
+     * get an ipaddress container iterator, and look for ipv6 addrs
+     */   
+    addr_it = CONTAINER_ITERATOR(addr_container);
+    if (NULL == addr_it) {
+        DEBUGMSGTL(("access:ifcontainer",
+                    "couldn't get ip addresses iterator\n"));
+        if (0!=addr_container_flags)
+            netsnmp_access_ipaddress_container_free(addr_container, 0);
+        return;
+    }
+
+    addr_entry = ITERATOR_FIRST(addr_it);
+    for( ; addr_entry ; addr_entry = ITERATOR_NEXT(addr_it) ) {
+        /*
+         * skip non matching indexes and ipv4 addresses
+         */
+        if ((if_index != addr_entry->if_index) ||
+            (4 == addr_entry->ia_address_len))
+            continue;
+
+        /*
+         * found one! no need to keep looking, set the flag and bail
+         */
+        *flags |= NETSNMP_INTERFACE_FLAGS_HAS_IPV6;
+        break;
+    }
+
+    /*
+     * make mama proud and clean up after ourselves
+     */
+    ITERATOR_RELEASE(addr_it);
+    if (0!=addr_container_flags)
+        netsnmp_access_ipaddress_container_free(addr_container, 0);    
+#endif
+}
+
+/**
+ * @internal
+ */
+static void
+_arch_interface_flags_v4_get(netsnmp_interface_entry *entry)
+{
+    FILE           *fin;
     char            line[256];
+
+    /*
+     * get the retransmit time
+     */
+    snprintf(line,sizeof(line), proc_sys_retrans_time, 4,
+             entry->name);
+    if (!(fin = fopen(line, "r"))) {
+        DEBUGMSGTL(("access:interface",
+                    "Failed to open %s\n", line));
+    }
+    else {
+        if (fgets(line, sizeof(line), fin)) {
+            entry->retransmit_v4 = atoi(line) * retrans_time_factor;
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_V4_RETRANSMIT;
+        }
+        fclose(fin);
+    }
+}
+
+
+#ifdef NETSNMP_ENABLE_IPV6
+/**
+ * @internal
+ */
+static void
+_arch_interface_flags_v6_get(netsnmp_interface_entry *entry)
+{
+    FILE           *fin;
+    char            line[256];
+
+    /*
+     * get the retransmit time
+     */
+    snprintf(line,sizeof(line), proc_sys_retrans_time, 6,
+             entry->name);
+    if (!(fin = fopen(line, "r"))) {
+        DEBUGMSGTL(("access:interface",
+                    "Failed to open %s\n", line));
+    }
+    else {
+        if (fgets(line, sizeof(line), fin)) {
+            entry->retransmit_v6 = atoi(line) * retrans_time_factor;
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_V6_RETRANSMIT;
+        }
+        fclose(fin);
+    }
+
+    /*
+     * get the forwarding status
+     */
+    snprintf(line, sizeof(line), "/proc/sys/net/ipv6/conf/%s/forwarding",
+             entry->name);
+    if (!(fin = fopen(line, "r"))) {
+        DEBUGMSGTL(("access:interface",
+                    "Failed to open %s\n", line));
+    }
+    else {
+        if (fgets(line, sizeof(line), fin)) {
+            entry->forwarding_v6 = atoi(line);
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_V6_FORWARDING;
+        }
+        fclose(fin);
+    }
+
+    /*
+     * get the reachable time
+     */
+    snprintf(line, sizeof(line), proc_sys_basereachable_time, 6, entry->name);
+    if (!(fin = fopen(line, "r"))) {
+        DEBUGMSGTL(("access:interface",
+                    "Failed to open %s\n", line));
+    }
+    else {
+        if (fgets(line, sizeof(line), fin)) {
+            if (basereachable_time_ms) {
+                entry->reachable_time = atoi(line); /* millisec */
+            } else {
+                entry->reachable_time = atoi(line)*1000; /* sec to  millisec */
+            }
+
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_V6_REACHABLE;
+        }
+        fclose(fin);
+    }
+}
+#endif /* NETSNMP_ENABLE_IPV6 */
+
+/**
+ * @internal
+ */
+static int
+_parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
+{
     /*
      * scanline_2_2:
      *  [               IN                        ]
@@ -104,9 +321,129 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         "%lu %lu %*lu %*lu %*lu %lu %lu %*lu %*lu %lu";
 #endif
     static const char     *scan_line_to_use = NULL;
-    static char     scan_expected;
-    int             scan_count, fd;
+    int             scan_count;
+
+    if (10 == expected)
+        scan_line_to_use = scan_line_2_2;
+    else {
+        netsnmp_assert(5 == expected);
+        scan_line_to_use = scan_line_2_0;
+    }
+
+    while (*stats == ' ')
+        stats++;
+
+    if ((*stats == 'N') &&
+        (0 == strncmp(stats, "No statistics available",
+                      sizeof("No statistics available"))))
+        return -1;
+
+    /*
+     * Now parse the rest of the line (i.e. starting from 'stats')
+     *      to extract the relevant statistics, and populate
+     *      data structure accordingly.
+     * Use the entry flags field to indicate which counters are valid
+     */
+    rec_pkt = rec_oct = rec_err = rec_drop = rec_mcast = 0;
+    snd_pkt = snd_oct = snd_err = snd_drop = coll = 0;
+    if (scan_line_to_use == scan_line_2_2) {
+        scan_count = sscanf(stats, scan_line_to_use,
+                            &rec_oct, &rec_pkt, &rec_err, &rec_drop, &rec_mcast,
+                            &snd_oct, &snd_pkt, &snd_err, &snd_drop,
+                            &coll);
+        if (scan_count == expected) {
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_BYTES;
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_DROPS;
+            /*
+             *  2.4 kernel includes a single multicast (input) counter?
+             */
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_MCAST_PKTS;
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_SPEED;
+#ifdef SCNuMAX   /* XXX - should be flag for 64-bit variables */
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_BYTES;
+            entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_PACKETS;
+#endif
+        }
+    } else {
+        scan_count = sscanf(stats, scan_line_to_use,
+                            &rec_pkt, &rec_err,
+                            &snd_pkt, &snd_err, &coll);
+        if (scan_count == expected) {
+            entry->ns_flags &= ~NETSNMP_INTERFACE_FLAGS_HAS_MCAST_PKTS;
+            rec_oct = rec_drop = 0;
+            snd_oct = snd_drop = 0;
+        }
+    }
+    if(scan_count != expected) {
+        snmp_log(LOG_ERR,
+                 "error scanning interface data (expected %d, got %d)\n",
+                 expected, scan_count);
+        return scan_count;
+    }
+    entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_ACTIVE;
+    
+    /*
+     * linux previous to 1.3.~13 may miss transmitted loopback pkts: 
+     */
+    if (!strcmp(entry->name, "lo") && rec_pkt > 0 && !snd_pkt)
+        snd_pkt = rec_pkt;
+    
+    /*
+     * subtract out multicast packets from rec_pkt before
+     * we store it as unicast counter.
+     */
+    rec_pkt -= rec_mcast;
+
+    entry->stats.ibytes.low = rec_oct & 0xffffffff;
+    entry->stats.iucast.low = rec_pkt & 0xffffffff;
+    entry->stats.imcast.low = rec_mcast & 0xffffffff;
+    entry->stats.obytes.low = snd_oct & 0xffffffff;
+    entry->stats.oucast.low = snd_pkt & 0xffffffff;
+#ifdef SCNuMAX   /* XXX - should be flag for 64-bit variables */
+    entry->stats.ibytes.high = rec_oct >> 32;
+    entry->stats.iucast.high = rec_pkt >> 32;
+    entry->stats.imcast.high = rec_mcast >> 32;
+    entry->stats.obytes.high = snd_oct >> 32;
+    entry->stats.oucast.high = snd_pkt >> 32;
+#endif
+    entry->stats.ierrors   = rec_err;
+    entry->stats.idiscards = rec_drop;
+    entry->stats.oerrors   = snd_err;
+    entry->stats.odiscards = snd_drop;
+    entry->stats.collisions = coll;
+    
+    /*
+     * calculated stats.
+     *
+     *  we have imcast, but not ibcast.
+     */
+    entry->stats.inucast = entry->stats.imcast.low +
+        entry->stats.ibcast.low;
+    entry->stats.onucast = entry->stats.omcast.low +
+        entry->stats.obcast.low;
+    
+    return 0;
+}
+
+/*
+ *
+ * @retval  0 success
+ * @retval -1 no container specified
+ * @retval -2 could not open /proc/net/dev
+ * @retval -3 could not create entry (probably malloc)
+ */
+int
+netsnmp_arch_interface_container_load(netsnmp_container* container,
+                                      u_int load_flags)
+{
+    FILE           *devin;
+    char            line[256];
     netsnmp_interface_entry *entry = NULL;
+    static char     scan_expected = 0;
+    int             fd;
+#ifdef NETSNMP_ENABLE_IPV6
+    netsnmp_container *addr_container;
+#endif
 
     DEBUGMSGTL(("access:interface:container:arch", "load (flags %p)\n",
                 load_flags));
@@ -132,6 +469,13 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         return -2;
     }
 
+#ifdef NETSNMP_ENABLE_IPV6
+    /*
+     * get ipv6 addresses
+     */
+    addr_container = netsnmp_access_ipaddress_container_load(NULL, 0);
+#endif
+
     /*
      * Read the first two lines of the file, containing the header
      * This indicates which version of the kernel we're working with,
@@ -143,17 +487,13 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
      */
     fgets(line, sizeof(line), devin);
     fgets(line, sizeof(line), devin);
-    /*
-     * XXX - What's the format for the 2.6 kernel ?
-     */
-    if( NULL == scan_line_to_use ) {
+
+    if( 0 == scan_expected ) {
         if (strstr(line, "compressed")) {
-            scan_line_to_use = scan_line_2_2;
             scan_expected = 10;
             DEBUGMSGTL(("access:interface",
                         "using linux 2.2 kernel /proc/net/dev\n"));
         } else {
-            scan_line_to_use = scan_line_2_0;
             scan_expected = 5;
             DEBUGMSGTL(("access:interface",
                         "using linux 2.0 kernel /proc/net/dev\n"));
@@ -167,7 +507,10 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
      */
     while (fgets(line, sizeof(line), devin)) {
         char           *stats, *ifstart = line;
+        u_int           flags;
+        oid             if_index;
 
+        flags = 0;
         if (line[strlen(line) - 1] == '\n')
             line[strlen(line) - 1] = '\0';
 
@@ -179,7 +522,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
                      "interface data format error 1, line ==|%s|\n", line);
             continue;
         }
-        if ((scan_line_to_use == scan_line_2_2) && ((stats - line) < 6)) {
+        if ((scan_expected == 10) && ((stats - line) < 6)) {
             snmp_log(LOG_ERR,
                      "interface data format error 2 (%d < 6), line ==|%s|\n",
                      stats - line, line);
@@ -193,106 +536,50 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
          * Otherwise find an unused index value and use that.
          */
         *stats++ = 0; /* null terminate name */
+
+        if_index = netsnmp_arch_interface_index_find(ifstart);
+
+        /*
+         * set address type flags.
+         * the only way I know of to check an interface for
+         * ip version is to look for ip addresses. If anyone
+         * knows a better way, put it here!
+         */
+#ifdef NETSNMP_ENABLE_IPV6
+        _arch_interface_has_ipv6(if_index, &flags, addr_container);
+#endif
+        netsnmp_access_interface_ioctl_has_ipv4(fd, ifstart, 0, &flags);
+
+        /*
+         * do we only want one address type?
+         */
+        if (((load_flags & NETSNMP_ACCESS_INTERFACE_LOAD_IP4_ONLY) &&
+             ((flags & NETSNMP_INTERFACE_FLAGS_HAS_IPV4) == 0)) ||
+            ((load_flags & NETSNMP_ACCESS_INTERFACE_LOAD_IP6_ONLY) &&
+             ((flags & NETSNMP_INTERFACE_FLAGS_HAS_IPV6) == 0))) {
+            DEBUGMSGTL(("9:access:ifcontainer",
+                        "interface '%s' excluded by ip version\n",
+                        ifstart));
+            continue;
+        }
+
         entry = netsnmp_access_interface_entry_create(ifstart, 0);
         if(NULL == entry) {
+#ifdef NETSNMP_ENABLE_IPV6
+            netsnmp_access_ipaddress_container_free(addr_container, 0);
+#endif
             netsnmp_access_interface_container_free(container,
                                                     NETSNMP_ACCESS_INTERFACE_FREE_NOFLAGS);
             fclose(devin);
             close(fd);
             return -3;
         }
-
-        /*
-         * OK - we've now got (or created) the data structure for
-         *      this interface, including any "static" information.
-         * Now parse the rest of the line (i.e. starting from 'stats')
-         *      to extract the relevant statistics, and populate
-         *      data structure accordingly.
-         * Use the ifentry flags field to indicate which counters are valid
-         */
-        while (*stats == ' ')
-            stats++;
-
-        rec_pkt = rec_oct = rec_err = rec_drop = rec_mcast = 0;
-        snd_pkt = snd_oct = snd_err = snd_drop = coll = 0;
-        if (scan_line_to_use == scan_line_2_2) {
-            scan_count = sscanf(stats, scan_line_to_use,
-                                &rec_oct, &rec_pkt, &rec_err, &rec_drop, &rec_mcast,
-                                &snd_oct, &snd_pkt, &snd_err, &snd_drop,
-                                &coll);
-            if (scan_count == scan_expected) {
-                entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_BYTES;
-                entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_DROPS;
-                /*
-                 *  2.4 kernel includes a single multicast (input) counter?
-                 */
-                entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_MCAST_PKTS;
-                entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_SPEED;
-#ifdef SCNuMAX   /* XXX - should be flag for 64-bit variables */
-                entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_BYTES;
-                entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_HIGH_PACKETS;
-#endif
-            }
-        } else {
-            scan_count = sscanf(stats, scan_line_to_use,
-                                &rec_pkt, &rec_err,
-                                &snd_pkt, &snd_err, &coll);
-            if (scan_count == scan_expected) {
-                entry->ns_flags &= ~NETSNMP_INTERFACE_FLAGS_HAS_MCAST_PKTS;
-                rec_oct = rec_drop = 0;
-                snd_oct = snd_drop = 0;
-            }
-        }
-        if(scan_count != scan_expected) {
-            snmp_log(LOG_ERR,
-                     "error scanning interface data (expected %d, got %d)\n",
-                     scan_expected, scan_count);
-            netsnmp_access_interface_entry_free(entry);
-            continue;
-        }
-        entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_ACTIVE;
-
-        /*
-         * linux previous to 1.3.~13 may miss transmitted loopback pkts: 
-         */
-        if (!strcmp(entry->name, "lo") && rec_pkt > 0 && !snd_pkt)
-            snd_pkt = rec_pkt;
+        entry->ns_flags = flags; /* initial flags; we'll set more later */
 
         /*
          * xxx-rks: get descr by linking mem from /proc/pci and /proc/iomem
          */
 
-        /*
-         * subtract out multicast packets from rec_pkt before
-         * we store it as unicast counter.
-         */
-        rec_pkt -= rec_mcast;
-
-        entry->stats.ibytes.low = rec_oct & 0xffffffff;
-        entry->stats.iucast.low = rec_pkt & 0xffffffff;
-        entry->stats.imcast.low = rec_mcast & 0xffffffff;
-        entry->stats.obytes.low = snd_oct & 0xffffffff;
-        entry->stats.oucast.low = snd_pkt & 0xffffffff;
-#ifdef SCNuMAX   /* XXX - should be flag for 64-bit variables */
-        entry->stats.ibytes.high = rec_oct >> 32;
-        entry->stats.iucast.high = rec_pkt >> 32;
-        entry->stats.imcast.high = rec_mcast >> 32;
-        entry->stats.obytes.high = snd_oct >> 32;
-        entry->stats.oucast.high = snd_pkt >> 32;
-#endif
-        entry->stats.ierrors   = rec_err;
-        entry->stats.idiscards = rec_drop;
-        entry->stats.oerrors   = snd_err;
-        entry->stats.odiscards = snd_drop;
-        entry->stats.collisions = coll;
-
-        /*
-         * calculated stats.
-         *
-         *  we have imcast, but not ibcast.
-         */
-        entry->stats.inucast = entry->stats.imcast.low +
-            entry->stats.ibcast.low;
 
         /*
          * use ioctls for some stuff
@@ -342,7 +629,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
 
         if (IANAIFTYPE_ETHERNETCSMACD == entry->type)
             entry->speed =
-                netsnmp_arch_interface_get_if_speed(fd, entry->name);
+                netsnmp_linux_interface_get_if_speed(fd, entry->name);
 #ifdef APPLIED_PATCH_836390   /* xxx-rks ifspeed fixes */
         else if (IANAIFTYPE_PROPVIRTUAL == entry->type)
             entry->speed = _get_bonded_if_speed(entry);
@@ -374,13 +661,38 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
             entry->promiscuous = 1; /* boolean */
         }
 
+        /*
+         * hardcoded max packet size
+         * (see ip_frag_reasm: if(len > 65535) goto out_oversize;)
+         */
+        entry->reasm_max_v4 = entry->reasm_max_v6 = 65535;
+        entry->ns_flags |= 
+            NETSNMP_INTERFACE_FLAGS_HAS_V4_REASMMAX |
+            NETSNMP_INTERFACE_FLAGS_HAS_V6_REASMMAX;
+
         netsnmp_access_interface_entry_overrides(entry);
 
+        entry->speed_high = entry->speed / 1000000;
+
+        if (! (load_flags & NETSNMP_ACCESS_INTERFACE_LOAD_NO_STATS))
+            _parse_stats(entry, stats, scan_expected);
+
+        if (flags & NETSNMP_INTERFACE_FLAGS_HAS_IPV4)
+            _arch_interface_flags_v4_get(entry);
+
+#ifdef NETSNMP_ENABLE_IPV6
+        if (flags & NETSNMP_INTERFACE_FLAGS_HAS_IPV6)
+            _arch_interface_flags_v6_get(entry);
+#endif /* NETSNMP_ENABLE_IPV6 */
+
         /*
          * add to container
          */
         CONTAINER_INSERT(container, entry);
     }
+#ifdef NETSNMP_ENABLE_IPV6
+    netsnmp_access_ipaddress_container_free(addr_container, 0);
+#endif
     fclose(devin);
     close(fd);
     return 0;
@@ -408,7 +720,7 @@ netsnmp_arch_set_admin_status(netsnmp_interface_entry * entry,
  * Determines network interface speed from ETHTOOL_GSET
  */
 unsigned int
-netsnmp_arch_interface_get_if_speed(int fd, const char *name)
+netsnmp_linux_interface_get_if_speed(int fd, const char *name)
 {
     struct ifreq ifr;
     struct ethtool_cmd edata;
@@ -422,7 +734,7 @@ netsnmp_arch_interface_get_if_speed(int fd, const char *name)
     if (ioctl(fd, SIOCETHTOOL, &ifr) == -1) {
         DEBUGMSGTL(("mibII/interfaces", "ETHTOOL_GSET on %s failed\n",
                     ifr.ifr_name));
-        return netsnmp_arch_interface_get_if_speed_mii(fd,name);
+        return netsnmp_linux_interface_get_if_speed_mii(fd,name);
     }
     
     if (edata.speed != SPEED_10 && edata.speed != SPEED_100 &&
@@ -430,7 +742,7 @@ netsnmp_arch_interface_get_if_speed(int fd, const char *name)
         DEBUGMSGTL(("mibII/interfaces", "fallback to mii for %s\n",
                     ifr.ifr_name));
         /* try MII */
-        return netsnmp_arch_interface_get_if_speed_mii(fd,name);
+        return netsnmp_linux_interface_get_if_speed_mii(fd,name);
     }
 
     /* return in bps */
@@ -445,9 +757,9 @@ netsnmp_arch_interface_get_if_speed(int fd, const char *name)
  */
 unsigned int
 #ifdef HAVE_LINUX_ETHTOOL_H
-netsnmp_arch_interface_get_if_speed_mii(int fd, const char *name)
+netsnmp_linux_interface_get_if_speed_mii(int fd, const char *name)
 #else
-netsnmp_arch_interface_get_if_speed(int fd, const char *name)
+netsnmp_linux_interface_get_if_speed(int fd, const char *name)
 #endif
 {
     unsigned int retspeed = 10000000;
