@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: ipaddress_ioctl.c,v 1.8.2.1 2005/02/09 21:02:14 nba Exp $
+ * $Id: ipaddress_ioctl.c 16381 2007-05-17 21:53:28Z hardaker $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -20,7 +20,6 @@
 
 #include "ipaddress_ioctl.h"
 
-static int _get_interface_count(int sd, struct ifconf * ifc);
 static void _print_flags(short flags);
 
 #define LIST_TOKEN "ioctl_extras"
@@ -135,6 +134,7 @@ _netsnmp_ioctl_ipaddress_container_load_v4(netsnmp_container *container,
     struct ifconf   ifc;
     struct ifreq   *ifrp;
     struct sockaddr save_addr;
+    struct sockaddr_in * si;
     netsnmp_ipaddress_entry *entry;
     _ioctl_extras           *extras;
 
@@ -143,7 +143,8 @@ _netsnmp_ioctl_ipaddress_container_load_v4(netsnmp_container *container,
         return -1;
     }
 
-    interfaces = _get_interface_count(sd, &ifc);
+    interfaces =
+        netsnmp_access_ipaddress_ioctl_get_interface_count(sd, &ifc);
     if(interfaces < 0) {
         close(sd);
         return -2;
@@ -180,32 +181,11 @@ _netsnmp_ioctl_ipaddress_container_load_v4(netsnmp_container *container,
         /*
          * set indexes
          */
-        switch(ifrp->ifr_addr.sa_family) {
-            case AF_INET: {
-                struct sockaddr_in * si =
-                    (struct sockaddr_in *) &ifrp->ifr_addr;
-                entry->ia_address_len = sizeof(si->sin_addr.s_addr);
-                memcpy(entry->ia_address, &si->sin_addr.s_addr,
-                       entry->ia_address_len);
-            }
-                break;
-                
-            case AF_INET6: {
-                struct sockaddr_in6 * si =
-                    (struct sockaddr_in6 *) &ifrp->ifr_addr;
-
-                entry->ia_address_len = sizeof(si->sin6_addr.s6_addr);
-                memcpy(entry->ia_address, &si->sin6_addr.s6_addr,
-                       entry->ia_address_len);
-            }
-                break;
-
-            default:
-                snmp_log(LOG_ERR,"unknown if family %d\n",
-                         ifrp->ifr_addr.sa_family);
-                netsnmp_access_ipaddress_entry_free(entry);
-                continue;
-        }
+        netsnmp_assert(AF_INET == ifrp->ifr_addr.sa_family);
+        si = (struct sockaddr_in *) &ifrp->ifr_addr;
+        entry->ia_address_len = sizeof(si->sin_addr.s_addr);
+        memcpy(entry->ia_address, &si->sin_addr.s_addr,
+               entry->ia_address_len);
 
         /*
          * get ifindex
@@ -232,6 +212,21 @@ _netsnmp_ioctl_ipaddress_container_load_v4(netsnmp_container *container,
             continue;
         }
 
+        /*
+         * get netmask
+         */
+        ifrp->ifr_addr = save_addr;
+        if (ioctl(sd, SIOCGIFNETMASK, ifrp) < 0) {
+            snmp_log(LOG_ERR,
+                     "error getting netmask for interface %d\n", i);
+            netsnmp_access_ipaddress_entry_free(entry);
+            continue;
+        }
+        netsnmp_assert(AF_INET == ifrp->ifr_addr.sa_family);
+        si = (struct sockaddr_in *) &ifrp->ifr_addr;
+        entry->ia_prefix_len =
+            netsnmp_ipaddress_ipv4_prefix_len(si->sin_addr.s_addr);
+
         /*
          * get flags
          */
@@ -277,7 +272,12 @@ _netsnmp_ioctl_ipaddress_container_load_v4(netsnmp_container *container,
         /*
          * add entry to container
          */
-        CONTAINER_INSERT(container, entry);
+        if (CONTAINER_INSERT(container, entry) < 0)
+        {
+            DEBUGMSGTL(("access:ipaddress:container","error with ipaddress_entry: insert into container failed.\n"));
+            netsnmp_access_ipaddress_entry_free(entry);
+            continue;
+        }
     }
 
     /*
@@ -316,7 +316,8 @@ _next_alias(const char *if_name)
         return -1;
     }
 
-    interfaces = _get_interface_count(sd, &ifc);
+    interfaces =
+        netsnmp_access_ipaddress_ioctl_get_interface_count(sd, &ifc);
     if(interfaces < 0) {
         close(sd);
         return -2;
@@ -484,15 +485,23 @@ _netsnmp_ioctl_ipaddress_delete_v4(netsnmp_ipaddress_entry * entry)
 }
 
 /**
+ * get the interface count and populate the ifc_buf
+ *
+ * Note: the caller assumes responsbility for the ifc->ifc_buf
+ *       memory, and should free() it when done.
  *
  * @retval -1 : malloc error
  */
-static int
-_get_interface_count(int sd, struct ifconf * ifc)
+int
+netsnmp_access_ipaddress_ioctl_get_interface_count(int sd, struct ifconf * ifc)
 {
     int lastlen = 0, i;
+    struct ifconf ifc_tmp;
 
-    netsnmp_assert(NULL != ifc);
+    if (NULL == ifc) {
+        memset(&ifc_tmp, 0x0, sizeof(ifc_tmp));
+        ifc = &ifc_tmp;
+    }
 
     /*
      * Cope with lots of interfaces and brokenness of ioctl SIOCGIFCONF
@@ -533,6 +542,9 @@ _get_interface_count(int sd, struct ifconf * ifc)
         free(ifc->ifc_buf); /* no SNMP_FREE, getting ready to reassign */
     }
 
+    if (ifc == &ifc_tmp)
+        free(ifc_tmp.ifc_buf);
+
     return ifc->ifc_len / sizeof(struct ifreq);
 }
 
