@@ -114,7 +114,7 @@
 #if HAVE_NETINET_IP_H
 #include <netinet/ip.h>
 #endif
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
 #if HAVE_NETINET_IP6_H
 #include <netinet/ip6.h>
 #endif
@@ -125,8 +125,8 @@
 #if HAVE_NETINET_IP_VAR_H
 #include <netinet/ip_var.h>
 #endif
-#ifdef INET6
-#if HAVE_NETINET6_IP6_VAR_H
+#ifdef NETSNMP_ENABLE_IPV6
+#if HAVE_NETNETSNMP_ENABLE_IPV6_IP6_VAR_H
 #include <netinet6/ip6_var.h>
 #endif
 #endif
@@ -190,7 +190,7 @@
 #if HAVE_OSRELDATE_H
 #include <osreldate.h>
 #endif
-#ifdef CAN_USE_SYSCTL
+#ifdef NETSNMP_CAN_USE_SYSCTL
 #include <sys/sysctl.h>
 #endif
 
@@ -651,7 +651,7 @@ var_ifEntry(struct variable *vp,
 #ifdef if_odrops
         long_return = (u_long) if_msg.ifm_data.ifi_odrops;
 #else
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = 0;
@@ -791,7 +791,7 @@ var_ifEntry(struct variable *vp,
             if (ifnet.if_type == IFT_ISDNPRIMARY)
                 long_return = 64000 * 30;
 #else
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
             return NULL;
 #endif
             long_return = (u_long) 10000000;
@@ -800,12 +800,16 @@ var_ifEntry(struct variable *vp,
         return (u_char *) & long_return;
     case IFPHYSADDRESS:
         Interface_Get_Ether_By_Index(interface, return_buf);
+#if defined(aix4) || defined(aix5)
+	*var_len = 0;
+#else
         if ((return_buf[0] == 0) && (return_buf[1] == 0) &&
             (return_buf[2] == 0) && (return_buf[3] == 0) &&
             (return_buf[4] == 0) && (return_buf[5] == 0))
             *var_len = 0;
         else
             *var_len = 6;
+#endif
         return (u_char *) return_buf;
     case IFADMINSTATUS:
         long_return = ifnet.if_flags & IFF_UP ? 1 : 2;
@@ -837,7 +841,7 @@ var_ifEntry(struct variable *vp,
                     starttime.tv_usec) / 10000);
         }
 #else
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = 0;        /* XXX */
@@ -845,9 +849,13 @@ var_ifEntry(struct variable *vp,
         return (u_char *) & long_return;
     case IFINOCTETS:
 #ifdef STRUCT_IFNET_HAS_IF_IBYTES
-        long_return = (u_long) ifnet.if_ibytes;
+#if defined(aix4) || defined(aix5)
+        long_return = (u_long) ifnet.if_ibytes & 0xffffffff;
 #else
-#if NO_DUMMY_VALUES
+        long_return = (u_long) ifnet.if_ibytes;
+#endif
+#else
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = (u_long) ifnet.if_ipackets * 308; /* XXX */
@@ -855,17 +863,29 @@ var_ifEntry(struct variable *vp,
         return (u_char *) & long_return;
     case IFINUCASTPKTS:
         {
+#if defined(aix4) || defined(aix5)
+            long_return = (u_long) ifnet.if_ipackets & 0xffffffff;
+#else
             long_return = (u_long) ifnet.if_ipackets;
+#endif
 #if STRUCT_IFNET_HAS_IF_IMCASTS
+#if defined(aix4) || defined(aix5)
+            long_return -= (u_long) ifnet.if_imcasts & 0xffffffff;
+#else
             long_return -= (u_long) ifnet.if_imcasts;
+#endif
 #endif
         }
         return (u_char *) & long_return;
     case IFINNUCASTPKTS:
 #if STRUCT_IFNET_HAS_IF_IMCASTS
-        long_return = (u_long) ifnet.if_imcasts;
+#if defined(aix4) || defined(aix5)
+        long_return = (u_long) ifnet.if_imcasts & 0xffffffff;
 #else
-#if NO_DUMMY_VALUES
+        long_return = (u_long) ifnet.if_imcasts;
+#endif
+#else
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = (u_long) 0;       /* XXX */
@@ -873,22 +893,34 @@ var_ifEntry(struct variable *vp,
         return (u_char *) & long_return;
     case IFINDISCARDS:
 #if STRUCT_IFNET_HAS_IF_IQDROPS
-        long_return = (u_long) ifnet.if_iqdrops;
+#if defined(aix4) || defined(aix5)
+        long_return = (u_long) ifnet.if_iqdrops & 0xffffffff;
 #else
-#if NO_DUMMY_VALUES
+        long_return = (u_long) ifnet.if_iqdrops;
+#endif
+#else
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = (u_long) 0;       /* XXX */
 #endif
         return (u_char *) & long_return;
     case IFINERRORS:
+#if defined(aix4) || defined(aix5)
+        long_return = (u_long) ifnet.if_ierrors & 0xffffffff;
+#else
         long_return = (u_long) ifnet.if_ierrors;
+#endif
         return (u_char *) & long_return;
     case IFINUNKNOWNPROTOS:
 #if STRUCT_IFNET_HAS_IF_NOPROTO
-        long_return = (u_long) ifnet.if_noproto;
+#if defined(aix4) || defined(aix5)
+        long_return = (u_long) ifnet.if_noproto & 0xffffffff;
 #else
-#if NO_DUMMY_VALUES
+        long_return = (u_long) ifnet.if_noproto;
+#endif
+#else
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = (u_long) 0;       /* XXX */
@@ -896,9 +928,13 @@ var_ifEntry(struct variable *vp,
         return (u_char *) & long_return;
     case IFOUTOCTETS:
 #ifdef STRUCT_IFNET_HAS_IF_OBYTES
-        long_return = (u_long) ifnet.if_obytes;
+#if defined(aix4) || defined(aix5)
+        long_return = (u_long) ifnet.if_obytes & 0xffffffff;
 #else
-#if NO_DUMMY_VALUES
+        long_return = (u_long) ifnet.if_obytes;
+#endif
+#else
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = (u_long) ifnet.if_opackets * 308; /* XXX */
@@ -906,30 +942,54 @@ var_ifEntry(struct variable *vp,
         return (u_char *) & long_return;
     case IFOUTUCASTPKTS:
         {
+#if defined(aix4) || defined(aix5)
+            long_return = (u_long) ifnet.if_opackets & 0xffffffff;
+#else
             long_return = (u_long) ifnet.if_opackets;
+#endif
 #if STRUCT_IFNET_HAS_IF_OMCASTS
+#if defined(aix4) || defined(aix5)
+            long_return -= (u_long) ifnet.if_omcasts & 0xffffffff;
+#else
             long_return -= (u_long) ifnet.if_omcasts;
+#endif
 #endif
         }
         return (u_char *) & long_return;
     case IFOUTNUCASTPKTS:
 #if STRUCT_IFNET_HAS_IF_OMCASTS
-        long_return = (u_long) ifnet.if_omcasts;
+#if defined(aix4) || defined(aix5)
+        long_return = (u_long) ifnet.if_omcasts & 0xffffffff;
 #else
-#if NO_DUMMY_VALUES
+        long_return = (u_long) ifnet.if_omcasts;
+#endif
+#else
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #endif
         long_return = (u_long) 0;       /* XXX */
 #endif
         return (u_char *) & long_return;
     case IFOUTDISCARDS:
+#if defined(aix4) || defined(aix5)
+        long_return = ifnet.if_snd.ifq_drops & 0xffffffff;
+#else
         long_return = ifnet.if_snd.ifq_drops;
+#endif
         return (u_char *) & long_return;
     case IFOUTERRORS:
+#if defined(aix4) || defined(aix5)
+        long_return = ifnet.if_oerrors & 0xffffffff;
+#else
         long_return = ifnet.if_oerrors;
+#endif
         return (u_char *) & long_return;
     case IFOUTQLEN:
+#if defined(aix4) || defined(aix5)
+        long_return = ifnet.if_snd.ifq_len & 0xffffffff;
+#else
         long_return = ifnet.if_snd.ifq_len;
+#endif
         return (u_char *) & long_return;
     case IFSPECIFIC:
         *var_len = nullOidLen;
@@ -1417,7 +1477,7 @@ Interface_Scan_Init(void)
 
 #endif
 
-#if !defined(hpux11)
+#if !defined(hpux11) && defined(IFNET_SYMBOL)
     auto_nlist(IFNET_SYMBOL, (char *) &ifnetaddr, sizeof(ifnetaddr));
 #endif
     saveIndex = 0;
@@ -1744,9 +1804,15 @@ Interface_Scan_Next(short *Index,
          *      Get the "ifnet" structure and extract the device name
          */
 #ifndef linux
-        klookup((unsigned long) ifnetaddr, (char *) &ifnet, sizeof ifnet);
-        klookup((unsigned long) ifnet.if_name, (char *) saveName,
-                sizeof saveName);
+        if (!NETSNMP_KLOOKUP(ifnetaddr, (char *) &ifnet, sizeof ifnet)) {
+            DEBUGMSGTL(("mibII/interfaces:Interface_Scan_Next", "klookup failed\n"));
+            break;
+        }
+
+        if (!NETSNMP_KLOOKUP(ifnet.if_name, (char *) saveName, sizeof saveName)) {
+            DEBUGMSGTL(("mibII/interfaces:Interface_Scan_Next", "klookup failed\n"));
+            break;
+        }
 
        /*
         * The purpose of this comparison is lost in the mists of time.
@@ -1866,18 +1932,25 @@ Interface_Scan_Next(short *Index,
         /*
          *      Get the "ifnet" structure and extract the device name
          */
-        klookup((unsigned long) ifnetaddr, (char *) &ifnet, sizeof ifnet);
+        if (!NETSNMP_KLOOKUP(ifnetaddr, (char *) &ifnet, sizeof ifnet)) {
+            DEBUGMSGTL(("mibII/interfaces:Interface_Scan_Next", "klookup failed\n"));
+            break;
+        }
 #if STRUCT_IFNET_HAS_IF_XNAME
 #if defined(netbsd1) || defined(openbsd2)
         strncpy(saveName, ifnet.if_xname, sizeof saveName);
 #else
-        klookup((unsigned long) ifnet.if_xname, (char *) saveName,
-                sizeof saveName);
+        if (!NETSNMP_KLOOKUP(ifnet.if_xname, (char *) saveName, sizeof saveName)) {
+            DEBUGMSGTL(("mibII/interfaces:Interface_Scan_Next", "klookup failed\n"));
+            break;
+        }
 #endif
         saveName[sizeof(saveName) - 1] = '\0';
 #else
-        klookup((unsigned long) ifnet.if_name, (char *) saveName,
-                sizeof saveName);
+        if (!NETSNMP_KLOOKUP(ifnet.if_name, (char *) saveName, sizeof saveName)) {
+            DEBUGMSGTL(("mibII/interfaces:Interface_Scan_Next", "klookup failed\n"));
+            break;
+        }
 
         saveName[sizeof(saveName) - 1] = '\0';
         cp = strchr(saveName, '\0');
@@ -1890,12 +1963,14 @@ Interface_Scan_Next(short *Index,
 
 #ifdef netbsd1
             ia = (struct in_ifaddr *) ifnet.if_addrlist.tqh_first;
-#else
+#elif defined(IFADDR_SYMBOL)
             auto_nlist(IFADDR_SYMBOL, (char *) &ia, sizeof(ia));
 #endif
             while (ia) {
-                klookup((unsigned long) ia, (char *) &in_ifaddr,
-                        sizeof(in_ifaddr));
+                if (!NETSNMP_KLOOKUP(ia, (char *) &in_ifaddr, sizeof(in_ifaddr))) {
+                    DEBUGMSGTL(("mibII/interfaces:Interface_Scan_Next", "klookup failed\n"));
+                    break;
+                }
                 {
 #ifdef netbsd1
 #define CP(x)	((char *)(x))
@@ -2087,8 +2162,10 @@ Interface_Get_Ether_By_Index(int Index, u_char * EtherAddr)
      */
 #ifndef linux
 #if !(defined(netbsd1) || defined(bsdi2) || defined(openbsd2))
-    klookup((unsigned long) saveifnetaddr, (char *) &arpcom,
-            sizeof arpcom);
+    if (!NETSNMP_KLOOKUP(saveifnetaddr, (char *) &arpcom, sizeof arpcom)) {
+        DEBUGMSGTL(("mibII/interfaces:Interface_Get_Ether_By_Index", "klookup failed\n"));
+        return 0;
+    }
 #else                           /* netbsd1 or bsdi2 or openbsd2 */
 
 #if defined(netbsd1) || defined(openbsd2)
@@ -2098,9 +2175,14 @@ Interface_Get_Ether_By_Index(int Index, u_char * EtherAddr)
 
     ifaddraddr = (unsigned long) saveifnet.if_addrlist;
     while (ifaddraddr) {
-        klookup(ifaddraddr, (char *) &ifaddr, sizeof ifaddr);
-        klookup((unsigned long) ifaddr.ifa_addr, (char *) &sadl,
-                sizeof sadl);
+        if (!NETSNMP_KLOOKUP(ifaddraddr, (char *) &ifaddr, sizeof ifaddr)) {
+            DEBUGMSGTL(("mibII/interfaces:Interface_Get_Ether_By_Index", "klookup failed\n"));
+            break;
+        }
+        if (!NETSNMP_KLOOKUP(ifaddr.ifa_addr, (char *) &sadl, sizeof sadl)) {
+            DEBUGMSGTL(("mibII/interfaces:Interface_Get_Ether_By_Index", "klookup failed\n"));
+            break;
+        }
         if (sadl.sdl_family == AF_LINK
             && (saveifnet.if_type == IFT_ETHER
                 || saveifnet.if_type == IFT_ISO88025
