@@ -31,6 +31,15 @@
 #endif
 #ifndef IP_STATS_CACHE_TIMEOUT
 #define IP_STATS_CACHE_TIMEOUT	MIB_STATS_CACHE_TIMEOUT
+#endif
+
+#if defined(HAVE_LIBPERFSTAT_H) && (defined(aix4) || defined(aix5)) && !defined(FIRST_PROTOCOL)
+#include <libperfstat.h>
+#ifdef FIRST_PROTOCOL
+perfstat_protocol_t ps_proto;
+perfstat_id_t ps_name;
+#define _USE_PERFSTAT_PROTOCOL 1
+#endif
 #endif
 
         /*********************
@@ -150,6 +159,7 @@ init_ip(void)
     /*
      * for speed optimization, we call this now to do the lookup 
      */
+#ifndef _USE_PERFSTAT_PROTOCOL
 #ifdef IPSTAT_SYMBOL
     auto_nlist(IPSTAT_SYMBOL, 0, 0);
 #endif
@@ -165,6 +175,7 @@ init_ip(void)
 #ifdef solaris2
     init_kernel_sunos5();
 #endif
+#endif
 }
 
 
@@ -235,9 +246,11 @@ ip_handler(netsnmp_mib_handler          *handler,
      *    cache handler, higher up the handler chain.
      * But just to be safe, check this and load it manually if necessary
      */
-#ifndef hpux11
+#ifdef _USE_PERFSTAT_PROTOCOL
+    ip_load(NULL, NULL);
+#elif !defined(hpux11)
     if (!netsnmp_cache_is_valid(reqinfo, reginfo->handlerName)) {
-        netsnmp_assert("cache" == "valid"); /* always false */
+        netsnmp_assert(!"cache == valid"); /* always false */
         ip_load( NULL, NULL );	/* XXX - check for failure */
     }
 #endif
@@ -324,9 +337,8 @@ ip_handler(netsnmp_mib_handler          *handler,
     case IPROUTEDISCARDS:
         ret_value = ipstat.ipRoutingDiscards;
         break;
-#else		/* USES_SNMP_DESIGNED_IPSTAT */
 
-#ifdef USES_TRADITIONAL_IPSTAT
+#elif defined(USES_TRADITIONAL_IPSTAT) && !defined(_USE_PERFSTAT_PROTOCOL)
 #ifdef HAVE_SYS_TCPIPSTATS_H
     /*
      * This actually reads statistics for *all* the groups together,
@@ -462,9 +474,7 @@ ip_handler(netsnmp_mib_handler          *handler,
 #ifdef HAVE_SYS_TCPIPSTATS_H
 #undef ipstat
 #endif
-#else                          /* USE_TRADITIONAL_IPSTAT */
-
-#ifdef hpux11
+#elif defined(hpux11)
     case IPFORWARDING:
     case IPDEFAULTTTL:
     case IPREASMTIMEOUT:
@@ -497,9 +507,7 @@ ip_handler(netsnmp_mib_handler          *handler,
 	}
         ret_value = ipstat;
         break;
-#else                  /* hpux11 */
-
-#if defined (WIN32) || defined (cygwin)
+#elif defined (WIN32) || defined (cygwin)
     case IPFORWARDING:
         ipForwarding = ipstat.dwForwarding;
         ret_value    = ipstat.dwForwarding;
@@ -565,9 +573,52 @@ ip_handler(netsnmp_mib_handler          *handler,
     case IPROUTEDISCARDS:
         ret_value = ipstat.dwRoutingDiscards;
         break;
-#endif                  /* WIN32 || cygwin */
-#endif                  /* hpux11 */
-#endif                  /* USE_TRADITIONAL_IPSTAT */
+#elif defined(_USE_PERFSTAT_PROTOCOL)
+    case IPFORWARDING:
+        ret_value    = 0;
+        type = ASN_INTEGER;
+        break;
+    case IPDEFAULTTTL:
+        ret_value = 0;
+        type = ASN_INTEGER;
+        break;
+    case IPINRECEIVES:
+        ret_value = ps_proto.u.ip.ipackets;
+        break;
+    case IPINHDRERRORS:
+    case IPINADDRERRORS:
+    case IPFORWDATAGRAMS:
+        ret_value = 0;
+        break;
+    case IPINUNKNOWNPROTOS:
+        ret_value = ps_proto.u.ip.ierrors;
+        break;
+    case IPINDISCARDS:
+        ret_value = 0;
+        break;
+    case IPINDELIVERS:
+    case IPOUTREQUESTS:
+        ret_value = ps_proto.u.ip.opackets;
+        break;
+    case IPOUTDISCARDS:
+    case IPOUTNOROUTES:
+        ret_value = 0;
+        break;
+    case IPREASMTIMEOUT:
+        ret_value = 0;
+        type = ASN_INTEGER;
+        break;
+    case IPREASMREQDS:
+    case IPREASMOKS:
+    case IPREASMFAILS:
+    case IPFRAGOKS:
+    case IPFRAGFAILS:
+    case IPFRAGCREATES:
+        ret_value = 0;
+        break;
+    case IPROUTEDISCARDS:
+        ret_value = ps_proto.u.ip.oerrors;
+        break;
 #endif			/* USES_SNMP_DESIGNED_IPSTAT */
 
     case IPADDRTABLE:
@@ -709,8 +760,7 @@ ip_load(netsnmp_cache *cache, void *vmagic)
                (ret < 0 ? "Failed to load" : "Loaded"),  magic));
     return (ret);         /* 0: ok, < 0: error */
 }
-#else                           /* hpux11 */
-#ifdef linux
+#elif defined(linux)
 int
 ip_load(netsnmp_cache *cache, void *vmagic)
 {
@@ -725,8 +775,7 @@ ip_load(netsnmp_cache *cache, void *vmagic)
     }
     return ret_value;
 }
-#else                           /* linux */
-#ifdef solaris2
+#elif defined(solaris2)
 int
 ip_load(netsnmp_cache *cache, void *vmagic)
 {
@@ -743,8 +792,7 @@ ip_load(netsnmp_cache *cache, void *vmagic)
     }
     return ret_value;
 }
-#else                           /* solaris2 */
-#if defined (WIN32) || defined (cygwin)
+#elif defined (WIN32) || defined (cygwin)
 int
 ip_load(netsnmp_cache *cache, void *vmagic)
 {
@@ -759,8 +807,24 @@ ip_load(netsnmp_cache *cache, void *vmagic)
     }
     return ret_value;
 }
-#else                           /* WIN32 cygwin */
-#if (defined(CAN_USE_SYSCTL) && defined(IPCTL_STATS))
+#elif defined(_USE_PERFSTAT_PROTOCOL)
+int
+ip_load(netsnmp_cache *cache, void *vmagic)
+{
+    long ret_value = -1;
+
+    strcpy(ps_name.name, "ip");
+    ret_value = perfstat_protocol(&ps_name, &ps_proto, sizeof(ps_proto), 1);
+
+    if ( ret_value < 0 ) {
+        DEBUGMSGTL(("mibII/ip", "Failed to load IP Group (AIX)\n"));
+    } else {
+        ret_value = 0;
+        DEBUGMSGTL(("mibII/ip", "Loaded IP Group (AIX)\n"));
+    }
+    return ret_value;
+}
+#elif defined(NETSNMP_CAN_USE_SYSCTL) && defined(IPCTL_STATS)
 int
 ip_load(netsnmp_cache *cache, void *vmagic)
 {
@@ -801,8 +865,7 @@ ip_load(netsnmp_cache *cache, void *vmagic)
         return ret_value;
     }
 }
-#else		/* (defined(CAN_USE_SYSCTL) && defined(IPCTL_STATS)) */
-#ifdef HAVE_SYS_TCPIPSTATS_H
+#elif defined(HAVE_SYS_TCPIPSTATS_H)
 int
 ip_load(netsnmp_cache *cache, void *vmagic)
 {
@@ -836,8 +899,7 @@ ip_load(netsnmp_cache *cache, void *vmagic)
         return ret_value;
     }
 }
-#else				/* HAVE_SYS_TCPIPSTATS_H */
-#ifdef IPSTAT_SYMBOL
+#elif defined(IPSTAT_SYMBOL)
 int
 ip_load(netsnmp_cache *cache, void *vmagic)
 {
@@ -881,17 +943,15 @@ ip_load(netsnmp_cache *cache, void *vmagic)
     DEBUGMSGTL(("mibII/ip", "Failed to load IP Group (null)\n"));
     return ret_value;
 }
-#endif				/* IPSTAT_SYMBOL */
-#endif				/* HAVE_SYS_TCPIPSTATS_H */
-#endif		/* (defined(CAN_USE_SYSCTL) && defined(IPCTL_STATS)) */
 #endif                          /* hpux11 */
-#endif                          /* linux */
-#endif                          /* solaris2 */
-#endif                          /* WIN32 cygwin */
 
 void
 ip_free(netsnmp_cache *cache, void *magic)
 {
+#if defined(_USE_PERFSTAT_PROTOCOL)
+    memset(&ps_proto, 0, sizeof(ps_proto));
+#else
     memset(&ipstat, 0, sizeof(ipstat));
+#endif
 }
 
