@@ -104,7 +104,7 @@
 #endif
 
 
-#ifdef EXCACHETIME
+#ifdef NETSNMP_EXCACHETIME
 static long     cachetime;
 #endif
 
@@ -117,6 +117,7 @@ Exit(int var)
     exit(var);
 }
 
+/** deprecated, use netsnmp_mktemp instead */
 const char *
 make_tempfile(void)
 {
@@ -221,12 +222,28 @@ get_exten_instance(struct extensible *exten, size_t inst)
 void
 wait_on_exec(struct extensible *ex)
 {
-#ifndef EXCACHETIME
+#if defined(WIN32) && !defined (mingw32)
+  int rc;
+  if (ex->tid != 0 && ex->pid != 0) {
+    HANDLE hThread = (HANDLE) ex->tid;
+    HANDLE hProcess = (HANDLE) ex->pid;
+    rc = WaitForSingleObject(hProcess, NETSNMP_TIMEOUT_WAITFORSINGLEOBJECT);
+    DEBUGMSGT(("exec:wait_on_exec","WaitForSingleObject rc=(%d)\n",rc ));
+    rc = CloseHandle( hThread );
+    DEBUGMSGT(("exec:wait_on_exec","CloseHandle hThread=(%d)\n",rc ));
+    rc = CloseHandle( hProcess );
+    DEBUGMSGT(("exec:wait_on_exec","CloseHandle hProcess=(%d)\n",rc ));
+    ex->pid = 0;
+    ex->tid = 0;
+  }
+#else
+#ifndef NETSNMP_EXCACHETIME
     if (ex->pid && waitpid(ex->pid, &ex->result, 0) < 0) {
         setPerrorstatus("waitpid");
     }
     ex->pid = 0;
-#endif
+#endif  /* NETSNMP_EXCACHETIME */
+#endif  /* WIN32 */
 }
 
 #define MAXARGS 30
@@ -236,10 +253,10 @@ get_exec_output(struct extensible *ex)
 {
 #if HAVE_EXECV
     char            cachefile[STRMAX];
-    char            cache[MAXCACHESIZE];
+    char            cache[NETSNMP_MAXCACHESIZE];
     ssize_t         cachebytes;
     int             cfd;
-#ifdef EXCACHETIME
+#ifdef NETSNMP_EXCACHETIME
     long            curtime;
     static char     lastcmd[STRMAX];
     static int      lastresult;
@@ -247,16 +264,16 @@ get_exec_output(struct extensible *ex)
 
     DEBUGMSGTL(("exec:get_exec_output","calling %s\n", ex->command));
 
-    sprintf(cachefile, "%s/%s", get_persistent_directory(), CACHEFILE);
-#ifdef EXCACHETIME
+    sprintf(cachefile, "%s/%s", get_persistent_directory(), NETSNMP_CACHEFILE);
+#ifdef NETSNMP_EXCACHETIME
     curtime = time(NULL);
-    if (curtime > (cachetime + EXCACHETIME) ||
+    if (curtime > (cachetime + NETSNMP_EXCACHETIME) ||
         strcmp(ex->command, lastcmd) != 0) {
         strcpy(lastcmd, ex->command);
         cachetime = curtime;
 #endif
 
-        cachebytes = MAXCACHESIZE;
+        cachebytes = NETSNMP_MAXCACHESIZE;
         ex->result = run_exec_command( ex->command, NULL, cache, &cachebytes );
 
         unlink(cachefile);
@@ -272,7 +289,7 @@ get_exec_output(struct extensible *ex)
         if (cachebytes > 0)
             write(cfd, (void *) cache, cachebytes);
         close(cfd);
-#ifdef EXCACHETIME
+#ifdef NETSNMP_EXCACHETIME
         lastresult = ex->result;
     } else {
         ex->result = lastresult;
@@ -361,6 +378,7 @@ get_exec_output(struct extensible *ex)
     
     /* Set global child process handle */
     ex->pid = (int)pi.hProcess;
+    ex->tid = (int)pi.hThread;
 
     /* Close pipe handles to make sure that no handles to the write end of the
      * output pipe are maintained in this process or else the pipe will
@@ -381,11 +399,65 @@ get_exec_output(struct extensible *ex)
     return -1;
 #endif
 }
-
 int
 get_exec_pipes(char *cmd, int *fdIn, int *fdOut, int *pid)
 {
+/* 	Alexander Prömel, alexander@proemel.de 08/24/2006
+	The following code, is tested on picotux rev. 1.01.
+	I think, it will be better to put the named pipes, into /var/run or make it selectable via CONFIG file.
+	If the pipe file already exist, the creation will fail.
+	I put the pipes into /flash, the pipepath has to change in ucd-snmp/pass_persist.c too, if you change it here.
+*/
 #if HAVE_EXECV
+#ifdef __uClinux__ /* HAVE uClinux */
+	int in,out;
+	char fifo_in_path[256];
+	char fifo_out_path[256];
+	pid_t tpid;
+        
+    if ((tpid = vfork()) == 0) { /*temp child*/
+        execve(cmd, NULL,NULL);
+        perror(cmd);
+        exit(1);
+    } else { 
+		if(tpid > 0) {
+			/*initialize workspace*/
+			snprintf(fifo_in_path, 256, "/flash/cp_%d", tpid);
+			snprintf(fifo_out_path, 256, "/flash/pc_%d", tpid);
+
+			in = mkfifo(fifo_in_path, S_IRWXU);	/*Create Input Pipe, 700*/
+			if ( in ) {
+				perror("parent: inpipe");
+				exit(0);
+			}
+			out = mkfifo(fifo_out_path, S_IRWXU);	/*Create Output Pipe, 700*/
+			if ( out ) {
+				perror("parent: outpipe");
+				exit(0);
+			}
+						
+			in = open(fifo_in_path,O_RDONLY);	/*open the Input Pipe read Only*/
+			if(in < 0) {
+				perror("parent: input");
+				exit(0);
+			}
+			out = open(fifo_out_path,O_WRONLY); 	/*open the Output Pipe write Only*/
+			if(out < 0) {
+				perror("parent: output");
+				exit(0);
+			}
+			
+			*fdIn = in;	/*read*/
+			*fdOut = out;	/*write*/
+			*pid = tpid;	
+			return (1);     /* We are returning 0 for error... */
+		} else { /*pid < 0*/
+			setPerrorstatus("vfork");
+			return 0;
+		}
+
+    }
+#else /*HAVE x86*/
     int             fd[2][2], i, cnt;
     char            ctmp[STRMAX], *cptr1, *cptr2, argvs[STRMAX], **argv,
         **aptr;
@@ -462,6 +534,7 @@ get_exec_pipes(char *cmd, int *fdIn, int *fdOut, int *pid)
         *fdOut = fd[0][1];
         return (1);             /* We are returning 0 for error... */
     }
+#endif				/* uClinux or x86 */
 #endif                          /* !HAVE_EXECV */
 #if defined(WIN32) && !defined (mingw32) && !defined(HAVE_EXECV)
 /* MSVC (MinGW not working but should use this code).  Cygwin already works as it has execv and fork */
@@ -598,7 +671,7 @@ clear_cache(int action,
     }
     tmp = *((long *) var_val);
     if (tmp == 1 && action == COMMIT) {
-#ifdef EXCACHETIME
+#ifdef NETSNMP_EXCACHETIME
         cachetime = 0;          /* reset the cache next read */
 #endif
     }
@@ -614,6 +687,17 @@ restart_doit(int a)
                                         NETSNMP_DS_LIB_APPTYPE);
     snmp_shutdown(name);
 
+    /*  This signal handler may run with SIGALARM blocked.
+     *  Since the signal mask is preserved accross execv(), we must 
+     *  make sure that SIGALARM is unblocked prior of execv'ing.
+     *  Otherwise SIGALARM will be ignored in the next incarnation
+     *  of snmpd, because the signal is blocked. And thus, the 
+     *  restart doesn't work anymore. 
+     */ 
+#if HAVE_SIGBLOCK 
+    sigsetmask(0);
+#endif 
+
     /*
      * do the exec 
      */
@@ -642,7 +726,7 @@ restart_hook(int action,
 #ifdef SIGALRM
         signal(SIGALRM, restart_doit);
 #endif
-        alarm(RESTARTSLEEP);
+        alarm(NETSNMP_RESTARTSLEEP);
     }
     return SNMP_ERR_NOERROR;
 }
@@ -856,7 +940,7 @@ find_field(char *ptr, int field)
     int             i;
     char           *init = ptr;
 
-    if (field == LASTFIELD) {
+    if (field == NETSNMP_LASTFIELD) {
         /*
          * skip to end 
          */
