@@ -1,11 +1,14 @@
 /*
  * tcpConn data access header
  *
- * $Id: tcpConn.h,v 1.4 2005/11/16 19:51:15 rstory Exp $
+ * $Id: tcpConn.h 15136 2006-09-01 17:50:29Z rstory $
  */
 #ifndef NETSNMP_ACCESS_TCPCONN_H
 #define NETSNMP_ACCESS_TCPCONN_H
 
+/** need def of NETSNMP_ACCESS_IPADDRESS_BUF_SIZE */
+#include <net-snmp/data_access/ipaddress.h>
+
 # ifdef __cplusplus
 extern          "C" {
 #endif
@@ -14,11 +17,6 @@ extern          "C" {
 /*
  * structure definitions
  */
-#if defined( INET6 )
-#   define NETSNMP_ACCESS_IPADDRESS_BUF_SIZE 20 /* for ip6z */
-#else
-#   define NETSNMP_ACCESS_IPADDRESS_BUF_SIZE 4
-#endif
 
 /*
  * netsnmp_tcpconn_entry
@@ -76,6 +74,7 @@ extern          "C" {
 #define NETSNMP_ACCESS_TCPCONN_LOAD_NOFLAGS               0x0000
 #define NETSNMP_ACCESS_TCPCONN_LOAD_NOLISTEN              0x0001
 #define NETSNMP_ACCESS_TCPCONN_LOAD_ONLYLISTEN            0x0002
+#define NETSNMP_ACCESS_TCPCONN_LOAD_IPV4_ONLY             0x0004
 
     void netsnmp_access_tcpconn_container_free(netsnmp_container *container,
                                                u_int free_flags);
