@@ -65,7 +65,7 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 	ASN_UNSIGNED
 	ASN_UNSIGNED64
 );
-$VERSION = '5.0203';
+$VERSION = '5.04011';
 
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
@@ -113,7 +113,7 @@ NetSNMP::ASN - Perl extension for SNMP ASN.1 types
 
 =head1 SYNOPSIS
 
-  use NetSNMP::ASN qw(':all');
+  use NetSNMP::ASN qw(:all);
   my $asn_int = ASN_INTEGER;
 
 =head1 DESCRIPTION
