@@ -53,11 +53,26 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 				   NETSNMP_DS_LIB_HAVE_READ_PREMIB_CONFIG
 				   NETSNMP_DS_LIB_HAVE_READ_CONFIG
 				   NETSNMP_DS_LIB_QUICKE_PRINT
+				   NETSNMP_DS_LIB_DONT_PRINT_UNITS
+				   NETSNMP_DS_LIB_NO_DISPLAY_HINT
+				   NETSNMP_DS_LIB_16BIT_IDS
+				   NETSNMP_DS_LIB_DONT_PERSIST_STATE
+				   NETSNMP_DS_LIB_2DIGIT_HEX_OUTPUT
+				   NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY
+				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_LOAD
+				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_SAVE
+				   NETSNMP_DS_LIB_APPEND_LOGFILES
 				   NETSNMP_DS_LIB_MIB_WARNINGS
 				   NETSNMP_DS_LIB_SECLEVEL
 				   NETSNMP_DS_LIB_SNMPVERSION
 				   NETSNMP_DS_LIB_DEFAULT_PORT
 				   NETSNMP_DS_LIB_OID_OUTPUT_FORMAT
+				   NETSNMP_DS_LIB_STRING_OUTPUT_FORMAT
+				   NETSNMP_DS_LIB_HEX_OUTPUT_LENGTH
+				   NETSNMP_DS_LIB_SERVERSENDBUF
+				   NETSNMP_DS_LIB_SERVERRECVBUF
+				   NETSNMP_DS_LIB_CLIENTSENDBUF
+				   NETSNMP_DS_LIB_CLIENTRECVBUF
 				   NETSNMP_DS_SNMP_VERSION_1
 				   NETSNMP_DS_SNMP_VERSION_2c
 				   NETSNMP_DS_SNMP_VERSION_3
@@ -73,6 +88,18 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 				   NETSNMP_DS_LIB_CONFIGURATION_DIR
 				   NETSNMP_DS_LIB_SECMODEL
 				   NETSNMP_DS_LIB_MIBDIRS
+				   NETSNMP_DS_LIB_OIDSUFFIX
+				   NETSNMP_DS_LIB_OIDPREFIX
+				   NETSNMP_DS_LIB_CLIENT_ADDR
+				   NETSNMP_DS_LIB_TEMP_FILE_PATTERN
+				   NETSNMP_DS_LIB_AUTHMASTERKEY
+				   NETSNMP_DS_LIB_PRIVMASTERKEY
+				   NETSNMP_DS_LIB_AUTHLOCALIZEDKEY
+				   NETSNMP_DS_LIB_PRIVLOCALIZEDKEY
+				   NETSNMP_DS_LIB_APPTYPES
+				   NETSNMP_DS_LIB_KSM_KEYTAB
+				   NETSNMP_DS_LIB_KSM_SERVICE_NAME
+				   NETSNMP_DS_LIB_SBSM_LOCAL_PWD
 	netsnmp_ds_get_boolean
 	netsnmp_ds_get_int
 	netsnmp_ds_get_string
@@ -124,11 +151,26 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 				   NETSNMP_DS_LIB_HAVE_READ_PREMIB_CONFIG
 				   NETSNMP_DS_LIB_HAVE_READ_CONFIG
 				   NETSNMP_DS_LIB_QUICKE_PRINT
+				   NETSNMP_DS_LIB_DONT_PRINT_UNITS
+				   NETSNMP_DS_LIB_NO_DISPLAY_HINT
+				   NETSNMP_DS_LIB_16BIT_IDS
+				   NETSNMP_DS_LIB_DONT_PERSIST_STATE
+				   NETSNMP_DS_LIB_2DIGIT_HEX_OUTPUT
+				   NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY
+				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_LOAD
+				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_SAVE
+				   NETSNMP_DS_LIB_APPEND_LOGFILES
 				   NETSNMP_DS_LIB_MIB_WARNINGS
 				   NETSNMP_DS_LIB_SECLEVEL
 				   NETSNMP_DS_LIB_SNMPVERSION
 				   NETSNMP_DS_LIB_DEFAULT_PORT
 				   NETSNMP_DS_LIB_OID_OUTPUT_FORMAT
+				   NETSNMP_DS_LIB_STRING_OUTPUT_FORMAT
+				   NETSNMP_DS_LIB_HEX_OUTPUT_LENGTH
+				   NETSNMP_DS_LIB_SERVERSENDBUF
+				   NETSNMP_DS_LIB_SERVERRECVBUF
+				   NETSNMP_DS_LIB_CLIENTSENDBUF
+				   NETSNMP_DS_LIB_CLIENTRECVBUF
 				   NETSNMP_DS_SNMP_VERSION_1
 				   NETSNMP_DS_SNMP_VERSION_2c
 				   NETSNMP_DS_SNMP_VERSION_3
@@ -144,36 +186,40 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 				   NETSNMP_DS_LIB_CONFIGURATION_DIR
 				   NETSNMP_DS_LIB_SECMODEL
 				   NETSNMP_DS_LIB_MIBDIRS
+				   NETSNMP_DS_LIB_OIDSUFFIX
+				   NETSNMP_DS_LIB_OIDPREFIX
+				   NETSNMP_DS_LIB_CLIENT_ADDR
+				   NETSNMP_DS_LIB_TEMP_FILE_PATTERN
+				   NETSNMP_DS_LIB_AUTHMASTERKEY
+				   NETSNMP_DS_LIB_PRIVMASTERKEY
+				   NETSNMP_DS_LIB_AUTHLOCALIZEDKEY
+				   NETSNMP_DS_LIB_PRIVLOCALIZEDKEY
+				   NETSNMP_DS_LIB_APPTYPES
+				   NETSNMP_DS_LIB_KSM_KEYTAB
+				   NETSNMP_DS_LIB_KSM_SERVICE_NAME
+				   NETSNMP_DS_LIB_SBSM_LOCAL_PWD
 );
-$VERSION = '5.0203';
+$VERSION = '5.04011';
 
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
-    # XS function.  If a constant is not found then control is passed
-    # to the AUTOLOAD in AutoLoader.
+    # XS function.
 
     my $constname;
     ($constname = $AUTOLOAD) =~ s/.*:://;
-    croak "& not defined" if $constname eq 'constant';
-    my $val = constant($constname, @_ ? $_[0] : 0);
-    if ($! != 0) {
-	if ($! =~ /Invalid/ || $!{EINVAL}) {
-	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
-	    goto &AutoLoader::AUTOLOAD;
-	}
-	else {
-	    croak "Your vendor has not defined NetSNMP::default_store macro $constname";
-	}
-    }
+    croak "&NetSNMP::default_store::constant not defined" if $constname eq 'cons
+tant';
+    my ($error, $val) = constant($constname);
+    if ($error) { croak $error; }
     {
-	no strict 'refs';
-	# Fixed between 5.005_53 and 5.005_61
-#	if ($] >= 5.00561) {
-#	    *$AUTOLOAD = sub () { $val };
-#	}
-#	else {
-	    *$AUTOLOAD = sub { $val };
-#	}
+        no strict 'refs';
+        # Fixed between 5.005_53 and 5.005_61
+#XXX    if ($] >= 5.00561) {
+#XXX        *$AUTOLOAD = sub () { $val };
+#XXX    }
+#XXX    else {
+            *$AUTOLOAD = sub { $val };
+#XXX    }
     }
     goto &$AUTOLOAD;
 }
@@ -243,11 +289,26 @@ None by default.
 				   NETSNMP_DS_LIB_HAVE_READ_PREMIB_CONFIG
 				   NETSNMP_DS_LIB_HAVE_READ_CONFIG
 				   NETSNMP_DS_LIB_QUICKE_PRINT
+				   NETSNMP_DS_LIB_DONT_PRINT_UNITS
+				   NETSNMP_DS_LIB_NO_DISPLAY_HINT
+				   NETSNMP_DS_LIB_16BIT_IDS
+				   NETSNMP_DS_LIB_DONT_PERSIST_STATE
+				   NETSNMP_DS_LIB_2DIGIT_HEX_OUTPUT
+				   NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY
+				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_LOAD
+				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_SAVE
+				   NETSNMP_DS_LIB_APPEND_LOGFILES
 				   NETSNMP_DS_LIB_MIB_WARNINGS
 				   NETSNMP_DS_LIB_SECLEVEL
 				   NETSNMP_DS_LIB_SNMPVERSION
 				   NETSNMP_DS_LIB_DEFAULT_PORT
 				   NETSNMP_DS_LIB_OID_OUTPUT_FORMAT
+				   NETSNMP_DS_LIB_STRING_OUTPUT_FORMAT
+				   NETSNMP_DS_LIB_HEX_OUTPUT_LENGTH
+				   NETSNMP_DS_LIB_SERVERSENDBUF
+				   NETSNMP_DS_LIB_SERVERRECVBUF
+				   NETSNMP_DS_LIB_CLIENTSENDBUF
+				   NETSNMP_DS_LIB_CLIENTRECVBUF
 				   NETSNMP_DS_SNMP_VERSION_1
 				   NETSNMP_DS_SNMP_VERSION_2c
 				   NETSNMP_DS_SNMP_VERSION_3
@@ -263,6 +324,18 @@ None by default.
 				   NETSNMP_DS_LIB_CONFIGURATION_DIR
 				   NETSNMP_DS_LIB_SECMODEL
 				   NETSNMP_DS_LIB_MIBDIRS
+				   NETSNMP_DS_LIB_OIDSUFFIX
+				   NETSNMP_DS_LIB_OIDPREFIX
+				   NETSNMP_DS_LIB_CLIENT_ADDR
+				   NETSNMP_DS_LIB_TEMP_FILE_PATTERN
+				   NETSNMP_DS_LIB_AUTHMASTERKEY
+				   NETSNMP_DS_LIB_PRIVMASTERKEY
+				   NETSNMP_DS_LIB_AUTHLOCALIZEDKEY
+				   NETSNMP_DS_LIB_PRIVLOCALIZEDKEY
+				   NETSNMP_DS_LIB_APPTYPES
+				   NETSNMP_DS_LIB_KSM_KEYTAB
+				   NETSNMP_DS_LIB_KSM_SERVICE_NAME
+				   NETSNMP_DS_LIB_SBSM_LOCAL_PWD
 
 =head2 Exportable functions
 
