@@ -44,9 +44,23 @@
 #include <net-snmp/library/snmpUDPDomain.h>
 #include <net-snmp/library/snmpTCPDomain.h>
 
+/* Copied from snmpUDPDomain.c */
+typedef struct netsnmp_udp_addr_pair_s {
+    struct sockaddr_in remote_addr;
+    struct in_addr local_addr;
+} netsnmp_udp_addr_pair;
+
 oid netsnmp_snmpTCPDomain[] = { TRANSPORT_DOMAIN_TCP_IP };
 static netsnmp_tdomain tcpDomain;
 
+/*
+ * Not static since it is needed here as well as in snmpUDPDomain, but not
+ * public either
+ */
+int
+netsnmp_sockaddr_in2(struct sockaddr_in *addr,
+                     const char *inpeername, const char *default_target);
+
 /*
  * Return a string representing the address in data, or else the "far end"
  * address if data is NULL.  
@@ -55,18 +69,23 @@ static netsnmp_tdomain tcpDomain;
 static char *
 netsnmp_tcp_fmtaddr(netsnmp_transport *t, void *data, int len)
 {
-    struct sockaddr_in *to = NULL;
+    netsnmp_udp_addr_pair *addr_pair = NULL;
 
-    if (data != NULL && len == sizeof(struct sockaddr_in)) {
-        to = (struct sockaddr_in *) data;
-    } else if (t != NULL && t->data != NULL &&
-               t->data_length == sizeof(struct sockaddr_in)) {
-        to = (struct sockaddr_in *) t->data;
+    if (data != NULL && len == sizeof(netsnmp_udp_addr_pair)) {
+	addr_pair = (netsnmp_udp_addr_pair *) data;
+    } else if (t != NULL && t->data != NULL) {
+	addr_pair = (netsnmp_udp_addr_pair *) t->data;
     }
-    if (to == NULL) {
+
+    if (addr_pair == NULL) {
         return strdup("TCP: unknown");
     } else {
-        char tmp[64];
+        struct sockaddr_in *to = NULL;
+	char tmp[64];
+        to = (struct sockaddr_in *) &(addr_pair->remote_addr);
+        if (to == NULL) {
+            return strdup("TCP: unknown");
+        }
 
         sprintf(tmp, "TCP: [%s]:%hd",
                 inet_ntoa(to->sin_addr), ntohs(to->sin_port));
@@ -163,19 +182,21 @@ static int
 netsnmp_tcp_accept(netsnmp_transport *t)
 {
     struct sockaddr *farend = NULL;
+    netsnmp_udp_addr_pair *addr_pair = NULL;
     int             newsock = -1, sockflags = 0;
     socklen_t       farendlen = sizeof(struct sockaddr_in);
     char           *str = NULL;
 
-    farend = (struct sockaddr *) malloc(sizeof(struct sockaddr_in));
+    addr_pair = (netsnmp_udp_addr_pair *)malloc(sizeof(netsnmp_udp_addr_pair));
 
-    if (farend == NULL) {
+    if (addr_pair == NULL) {
         /*
          * Indicate that the acceptance of this socket failed.  
          */
         DEBUGMSGTL(("netsnmp_tcp", "accept: malloc failed\n"));
         return -1;
     }
+    farend = (struct sockaddr *) &(addr_pair->remote_addr);
 
     if (t != NULL && t->sock >= 0) {
         newsock = accept(t->sock, farend, &farendlen);
@@ -191,8 +212,8 @@ netsnmp_tcp_accept(netsnmp_transport *t)
             free(t->data);
         }
 
-        t->data = farend;
-        t->data_length = farendlen;
+        t->data = addr_pair;
+        t->data_length = sizeof(netsnmp_udp_addr_pair);
         str = netsnmp_tcp_fmtaddr(NULL, farend, farendlen);
         DEBUGMSGTL(("netsnmp_tcp", "accept succeeded (from %s)\n", str));
         free(str);
@@ -238,6 +259,7 @@ netsnmp_transport *
 netsnmp_tcp_transport(struct sockaddr_in *addr, int local)
 {
     netsnmp_transport *t = NULL;
+    netsnmp_udp_addr_pair *addr_pair = NULL;
     int rc = 0;
 
 
@@ -251,13 +273,14 @@ netsnmp_tcp_transport(struct sockaddr_in *addr, int local)
     }
     memset(t, 0, sizeof(netsnmp_transport));
 
-    t->data = malloc(sizeof(struct sockaddr_in));
-    if (t->data == NULL) {
+    addr_pair = (netsnmp_udp_addr_pair *)malloc(sizeof(netsnmp_udp_addr_pair));
+    if (addr_pair == NULL) {
         netsnmp_transport_free(t);
         return NULL;
     }
-    t->data_length = sizeof(struct sockaddr_in);
-    memcpy(t->data, addr, sizeof(struct sockaddr_in));
+    t->data = addr_pair;
+    t->data_length = sizeof(netsnmp_udp_addr_pair);
+    memcpy(&(addr_pair->remote_addr), addr, sizeof(struct sockaddr_in));
 
     t->domain = netsnmp_snmpTCPDomain;
     t->domain_length =
@@ -281,7 +304,7 @@ netsnmp_tcp_transport(struct sockaddr_in *addr, int local)
          */
 
         t->flags |= NETSNMP_TRANSPORT_FLAG_LISTEN;
-        t->local = malloc(6);
+        t->local = (u_char *)malloc(6);
         if (t->local == NULL) {
             netsnmp_tcp_close(t);
             netsnmp_transport_free(t);
@@ -338,7 +361,7 @@ netsnmp_tcp_transport(struct sockaddr_in *addr, int local)
          */
 
     } else {
-        t->remote = malloc(6);
+      t->remote = (u_char *)malloc(6);
         if (t->remote == NULL) {
             netsnmp_tcp_close(t);
             netsnmp_transport_free(t);
@@ -392,11 +415,12 @@ netsnmp_tcp_transport(struct sockaddr_in *addr, int local)
 
 
 netsnmp_transport *
-netsnmp_tcp_create_tstring(const char *str, int local)
+netsnmp_tcp_create_tstring(const char *str, int local,
+			   const char *default_target)
 {
     struct sockaddr_in addr;
 
-    if (netsnmp_sockaddr_in(&addr, str, 0)) {
+    if (netsnmp_sockaddr_in2(&addr, str, default_target)) {
         return netsnmp_tcp_transport(&addr, local);
     } else {
         return NULL;
@@ -427,10 +451,10 @@ netsnmp_tcp_ctor(void)
 {
     tcpDomain.name = netsnmp_snmpTCPDomain;
     tcpDomain.name_length = sizeof(netsnmp_snmpTCPDomain) / sizeof(oid);
-    tcpDomain.prefix = calloc(2, sizeof(char *));
+    tcpDomain.prefix = (const char **)calloc(2, sizeof(char *));
     tcpDomain.prefix[0] = "tcp";
 
-    tcpDomain.f_create_from_tstring = netsnmp_tcp_create_tstring;
+    tcpDomain.f_create_from_tstring_new = netsnmp_tcp_create_tstring;
     tcpDomain.f_create_from_ostring = netsnmp_tcp_create_ostring;
 
     netsnmp_tdomain_register(&tcpDomain);
