@@ -39,6 +39,9 @@
 #if HAVE_NETDB_H
 #include <netdb.h>
 #endif
+#if HAVE_SYS_UIO_H
+#include <sys/uio.h>
+#endif
 
 #if HAVE_WINSOCK_H
 #include <winsock2.h>
@@ -64,11 +67,19 @@
 
 static netsnmp_tdomain udpDomain;
 
+typedef struct netsnmp_udp_addr_pair_s {
+    struct sockaddr_in remote_addr;
+    struct in_addr local_addr;
+} netsnmp_udp_addr_pair;
+
 /*
  * not static, since snmpUDPIPv6Domain needs it, but not public, either.
  * (ie don't put it in a public header.)
  */
 void _netsnmp_udp_sockopt_set(int fd, int server);
+int
+netsnmp_sockaddr_in2(struct sockaddr_in *addr,
+                     const char *inpeername, const char *default_target);
 
 /*
  * Return a string representing the address in data, or else the "far end"
@@ -78,17 +89,23 @@ void _netsnmp_udp_sockopt_set(int fd, int server);
 static char *
 netsnmp_udp_fmtaddr(netsnmp_transport *t, void *data, int len)
 {
-    struct sockaddr_in *to = NULL;
+    netsnmp_udp_addr_pair *addr_pair = NULL;
 
-    if (data != NULL && len == sizeof(struct sockaddr_in)) {
-        to = (struct sockaddr_in *) data;
+    if (data != NULL && len == sizeof(netsnmp_udp_addr_pair)) {
+	addr_pair = (netsnmp_udp_addr_pair *) data;
     } else if (t != NULL && t->data != NULL) {
-        to = (struct sockaddr_in *) t->data;
+	addr_pair = (netsnmp_udp_addr_pair *) t->data;
     }
-    if (to == NULL) {
+
+    if (addr_pair == NULL) {
         return strdup("UDP: unknown");
     } else {
+        struct sockaddr_in *to = NULL;
 	char tmp[64];
+        to = (struct sockaddr_in *) &(addr_pair->remote_addr);
+        if (to == NULL) {
+            return strdup("UDP: unknown");
+        }
 
         sprintf(tmp, "UDP: [%s]:%hu",
                 inet_ntoa(to->sin_addr), ntohs(to->sin_port));
@@ -98,6 +115,74 @@ netsnmp_udp_fmtaddr(netsnmp_transport *t, void *data, int len)
 
 
 
+#if defined(linux) && defined(IP_PKTINFO)
+
+# define netsnmp_dstaddr(x) (&(((struct in_pktinfo *)(CMSG_DATA(x)))->ipi_addr))
+
+static int netsnmp_udp_recvfrom(int s, void *buf, int len, struct sockaddr *from, socklen_t *fromlen, struct in_addr *dstip)
+{
+    int r;
+    struct iovec iov[1];
+    char cmsg[CMSG_SPACE(sizeof(struct in_pktinfo))];
+    struct cmsghdr *cmsgptr;
+    struct msghdr msg;
+
+    iov[0].iov_base = buf;
+    iov[0].iov_len = len;
+
+    memset(&msg, 0, sizeof msg);
+    msg.msg_name = from;
+    msg.msg_namelen = *fromlen;
+    msg.msg_iov = iov;
+    msg.msg_iovlen = 1;
+    msg.msg_control = &cmsg;
+    msg.msg_controllen = sizeof(cmsg);
+
+    r = recvmsg(s, &msg, 0);
+
+    if (r == -1) {
+        return -1;
+    }
+    
+    DEBUGMSGTL(("netsnmp_udp", "got source addr: %s\n", inet_ntoa(((struct sockaddr_in *)from)->sin_addr)));
+    for (cmsgptr = CMSG_FIRSTHDR(&msg); cmsgptr != NULL; cmsgptr = CMSG_NXTHDR(&msg, cmsgptr)) {
+        if (cmsgptr->cmsg_level == SOL_IP && cmsgptr->cmsg_type == IP_PKTINFO) {
+            memcpy((void *) dstip, netsnmp_dstaddr(cmsgptr), sizeof(struct in_addr));
+            DEBUGMSGTL(("netsnmp_udp", "got destination (local) addr %s\n",
+                    inet_ntoa(*dstip)));
+        }
+    }
+    return r;
+}
+
+static int netsnmp_udp_sendto(int fd, struct in_addr *srcip, struct sockaddr *remote,
+			void *data, int len)
+{
+    struct iovec iov = { data, len };
+    struct {
+        struct cmsghdr cm;
+        struct in_pktinfo ipi;
+    } cmsg;
+    struct msghdr m;
+
+    cmsg.cm.cmsg_len = sizeof(struct cmsghdr) + sizeof(struct in_pktinfo);
+    cmsg.cm.cmsg_level = SOL_IP;
+    cmsg.cm.cmsg_type = IP_PKTINFO;
+    cmsg.ipi.ipi_ifindex = 0;
+    cmsg.ipi.ipi_spec_dst.s_addr = (srcip ? srcip->s_addr : INADDR_ANY);
+
+    m.msg_name		= remote;
+    m.msg_namelen	= sizeof(struct sockaddr_in);
+    m.msg_iov		= &iov;
+    m.msg_iovlen	= 1;
+    m.msg_control	= &cmsg;
+    m.msg_controllen	= sizeof(cmsg);
+    m.msg_flags		= 0;
+
+    return sendmsg(fd, &m, MSG_NOSIGNAL|MSG_DONTWAIT);
+}
+#endif /* linux && IP_PKTINFO */
+
 /*
  * You can write something into opaque that will subsequently get passed back 
  * to your send function if you like.  For instance, you might want to
@@ -110,27 +195,33 @@ netsnmp_udp_recv(netsnmp_transport *t, void *buf, int size,
 {
     int             rc = -1;
     socklen_t       fromlen = sizeof(struct sockaddr);
+    netsnmp_udp_addr_pair *addr_pair = NULL;
     struct sockaddr *from;
 
     if (t != NULL && t->sock >= 0) {
-        from = (struct sockaddr *) malloc(sizeof(struct sockaddr_in));
-        if (from == NULL) {
+        addr_pair = (netsnmp_udp_addr_pair *) malloc(sizeof(netsnmp_udp_addr_pair));
+        if (addr_pair == NULL) {
             *opaque = NULL;
             *olength = 0;
             return -1;
         } else {
-            memset(from, 0, fromlen);
+            memset(addr_pair, 0, sizeof(netsnmp_udp_addr_pair));
+            from = (struct sockaddr *) &(addr_pair->remote_addr);
         }
 
 	while (rc < 0) {
-	    rc = recvfrom(t->sock, buf, size, 0, from, &fromlen);
+#if defined(linux) && defined(IP_PKTINFO)
+            rc = netsnmp_udp_recvfrom(t->sock, buf, size, from, &fromlen, &(addr_pair->local_addr));
+#else
+            rc = recvfrom(t->sock, buf, size, 0, from, &fromlen);
+#endif /* linux && IP_PKTINFO */
 	    if (rc < 0 && errno != EINTR) {
 		break;
 	    }
 	}
 
         if (rc >= 0) {
-            char *str = netsnmp_udp_fmtaddr(NULL, from, fromlen);
+            char *str = netsnmp_udp_fmtaddr(NULL, addr_pair, sizeof(netsnmp_udp_addr_pair));
             DEBUGMSGTL(("netsnmp_udp",
 			"recvfrom fd %d got %d bytes (from %s)\n",
 			t->sock, rc, str));
@@ -139,8 +230,8 @@ netsnmp_udp_recv(netsnmp_transport *t, void *buf, int size,
             DEBUGMSGTL(("netsnmp_udp", "recvfrom fd %d err %d (\"%s\")\n",
                         t->sock, errno, strerror(errno)));
         }
-        *opaque = (void *)from;
-        *olength = sizeof(struct sockaddr_in);
+        *opaque = (void *)addr_pair;
+        *olength = sizeof(netsnmp_udp_addr_pair);
     }
     return rc;
 }
@@ -152,24 +243,31 @@ netsnmp_udp_send(netsnmp_transport *t, void *buf, int size,
 		 void **opaque, int *olength)
 {
     int rc = -1;
+    netsnmp_udp_addr_pair *addr_pair = NULL;
     struct sockaddr *to = NULL;
 
     if (opaque != NULL && *opaque != NULL &&
-        *olength == sizeof(struct sockaddr_in)) {
-        to = (struct sockaddr *) (*opaque);
+        *olength == sizeof(netsnmp_udp_addr_pair)) {
+        addr_pair = (netsnmp_udp_addr_pair *) (*opaque);
     } else if (t != NULL && t->data != NULL &&
-               t->data_length == sizeof(struct sockaddr_in)) {
-        to = (struct sockaddr *) (t->data);
+                t->data_length == sizeof(netsnmp_udp_addr_pair)) {
+        addr_pair = (netsnmp_udp_addr_pair *) (t->data);
     }
 
+    to = (struct sockaddr *) &(addr_pair->remote_addr);
+
     if (to != NULL && t != NULL && t->sock >= 0) {
-        char *str = netsnmp_udp_fmtaddr(NULL, (void *) to,
-					sizeof(struct sockaddr_in));
+        char *str = netsnmp_udp_fmtaddr(NULL, (void *) addr_pair,
+                                        sizeof(netsnmp_udp_addr_pair));
         DEBUGMSGTL(("netsnmp_udp", "send %d bytes from %p to %s on fd %d\n",
                     size, buf, str, t->sock));
         free(str);
 	while (rc < 0) {
-	    rc = sendto(t->sock, buf, size, 0, to, sizeof(struct sockaddr));
+#if defined(linux) && defined(IP_PKTINFO)
+            rc = netsnmp_udp_sendto(t->sock, addr_pair ? &(addr_pair->local_addr) : NULL, to, buf, size);
+#else
+            rc = sendto(t->sock, buf, size, 0, to, sizeof(struct sockaddr));
+#endif /* linux && IP_PKTINFO */
 	    if (rc < 0 && errno != EINTR) {
                 DEBUGMSGTL(("netsnmp_udp", "sendto error, rc %d (errno %d)\n",
                             rc, errno));
@@ -207,7 +305,8 @@ netsnmp_udp_close(netsnmp_transport *t)
 static int
 _sock_buffer_maximize(int s, int optname, const char *buftype, int size)
 {
-    int            curbuf = 0, curbuflen = sizeof(int);
+    int            curbuf = 0;
+    size_t         curbuflen = sizeof(int);
     int            lo, mid, hi;
 
     /*
@@ -309,32 +408,32 @@ _sock_buffer_size_get(int optname, int local, const char **buftype)
         if (local) {
             size = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
                     NETSNMP_DS_LIB_SERVERSENDBUF);
-#ifdef DEFAULT_SERVER_SEND_BUF
+#ifdef NETSNMP_DEFAULT_SERVER_SEND_BUF
             if (size <= 0)
-               size = DEFAULT_SERVER_SEND_BUF;
+               size = NETSNMP_DEFAULT_SERVER_SEND_BUF;
 #endif
         } else {
             size = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
                     NETSNMP_DS_LIB_CLIENTSENDBUF);
-#ifdef DEFAULT_CLIENT_SEND_BUF
+#ifdef NETSNMP_DEFAULT_CLIENT_SEND_BUF
             if (size <= 0)
-               size = DEFAULT_CLIENT_SEND_BUF;
+               size = NETSNMP_DEFAULT_CLIENT_SEND_BUF;
 #endif
         }
     } else if (optname == SO_RCVBUF) {
         if (local) {
             size = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
                     NETSNMP_DS_LIB_SERVERRECVBUF);
-#ifdef DEFAULT_SERVER_RECV_BUF
+#ifdef NETSNMP_DEFAULT_SERVER_RECV_BUF
             if (size <= 0)
-               size = DEFAULT_SERVER_RECV_BUF;
+               size = NETSNMP_DEFAULT_SERVER_RECV_BUF;
 #endif
         } else {
             size = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
                     NETSNMP_DS_LIB_CLIENTRECVBUF);
-#ifdef DEFAULT_CLIENT_RECV_BUF
+#ifdef NETSNMP_DEFAULT_CLIENT_RECV_BUF
             if (size <= 0)
-               size = DEFAULT_CLIENT_RECV_BUF;
+               size = NETSNMP_DEFAULT_CLIENT_RECV_BUF;
 #endif
         }
     } else {
@@ -366,7 +465,8 @@ netsnmp_sock_buffer_set(int s, int optname, int local, int size)
     return -1;
 #else
     const char     *buftype;
-    int            curbuf = 0, curbuflen = sizeof(int);
+    int            curbuf = 0;
+    size_t         curbuflen = sizeof(int);
 
 #   ifndef  SO_SNDBUF
     if (SO_SNDBUF == optname) {
@@ -486,20 +586,24 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
     int             rc = 0;
     char           *str = NULL;
     char           *client_socket = NULL;
+    netsnmp_udp_addr_pair addr_pair;
 
     if (addr == NULL || addr->sin_family != AF_INET) {
         return NULL;
     }
 
+    memset(&addr_pair, 0, sizeof(netsnmp_udp_addr_pair));
+    memcpy(&(addr_pair.remote_addr), addr, sizeof(struct sockaddr_in));
+
     t = (netsnmp_transport *) malloc(sizeof(netsnmp_transport));
     if (t == NULL) {
         return NULL;
     }
 
-    str = netsnmp_udp_fmtaddr(NULL, (void *)addr, 
-				 sizeof(struct sockaddr_in));
-    DEBUGMSGTL(("netsnmp_udp", "open %s %s:%d\n", local ? "local" : "remote",
-                str,addr->sin_port));
+    str = netsnmp_udp_fmtaddr(NULL, (void *)&addr_pair,
+                                 sizeof(netsnmp_udp_addr_pair));
+    DEBUGMSGTL(("netsnmp_udp", "open %s %s\n", local ? "local" : "remote",
+                str));
     free(str);
 
     memset(t, 0, sizeof(netsnmp_transport));
@@ -522,7 +626,7 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
          * be INADDR_ANY, but certainly includes a port number.
          */
 
-        t->local = malloc(6);
+      t->local = (u_char *) malloc(6);
         if (t->local == NULL) {
             netsnmp_transport_free(t);
             return NULL;
@@ -532,6 +636,17 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
         t->local[5] = (htons(addr->sin_port) & 0x00ff) >> 0;
         t->local_length = 6;
 
+#if defined(linux) && defined(IP_PKTINFO)
+        { 
+            int sockopt = 1;
+            if (setsockopt(t->sock, SOL_IP, IP_PKTINFO, &sockopt, sizeof sockopt) == -1) {
+                DEBUGMSGTL(("netsnmp_udp", "couldn't set IP_PKTINFO: %s\n",
+                    strerror(errno)));
+                return NULL;
+            }
+            DEBUGMSGTL(("netsnmp_udp", "set IP_PKTINFO\n"));
+        }
+#endif
         rc = bind(t->sock, (struct sockaddr *) addr,
                   sizeof(struct sockaddr));
         if (rc != 0) {
@@ -551,7 +666,7 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
                                               NETSNMP_DS_LIB_CLIENT_ADDR);
         if (client_socket) {
             struct sockaddr_in client_addr;
-            netsnmp_sockaddr_in( &client_addr, client_socket, 0);
+            netsnmp_sockaddr_in2(&client_addr, client_socket, NULL);
             client_addr.sin_port = 0;
             bind(t->sock, (struct sockaddr *)&client_addr,
                   sizeof(struct sockaddr));
@@ -561,8 +676,8 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
          * transport-specific data pointer for later use by netsnmp_udp_send.
          */
 
-        t->data = malloc(sizeof(struct sockaddr_in));
-        t->remote = malloc(6);
+        t->data = malloc(sizeof(netsnmp_udp_addr_pair));
+        t->remote = (u_char *)malloc(6);
         if (t->data == NULL || t->remote == NULL) {
             netsnmp_transport_free(t);
             return NULL;
@@ -571,8 +686,8 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
         t->remote[4] = (htons(addr->sin_port) & 0xff00) >> 8;
         t->remote[5] = (htons(addr->sin_port) & 0x00ff) >> 0;
         t->remote_length = 6;
-        memcpy(t->data, addr, sizeof(struct sockaddr_in));
-        t->data_length = sizeof(struct sockaddr_in);
+        memcpy(t->data, &addr_pair, sizeof(netsnmp_udp_addr_pair));
+        t->data_length = sizeof(netsnmp_udp_addr_pair);
     }
 
     /*
@@ -639,32 +754,48 @@ _netsnmp_udp_sockopt_set(int fd, int local)
 }
 
 int
-netsnmp_sockaddr_in(struct sockaddr_in *addr,
-                    const char *inpeername, int remote_port)
+netsnmp_sockaddr_in2(struct sockaddr_in *addr,
+                     const char *inpeername, const char *default_target)
 {
-    char           *cp = NULL, *peername = NULL;
+#if HAVE_GETADDRINFO
+    struct addrinfo *addrs = NULL;
+    struct addrinfo hint;
+    int             err;
+#elif HAVE_GETIPNODEBYNAME
+    struct hostent *hp = NULL;
+    int             err;
+#elif HAVE_GETHOSTBYNAME
+    struct hostent *hp = NULL;
+#endif
 
     if (addr == NULL) {
         return 0;
     }
+
+    DEBUGMSGTL(("netsnmp_sockaddr_in",
+                "addr %p, inpeername \"%s\", default_target \"%s\"\n",
+                addr, inpeername ? inpeername : "[NIL]",
+                default_target ? default_target : "[NIL]"));
+
     memset(addr, 0, sizeof(struct sockaddr_in));
-
-    DEBUGMSGTL(("netsnmp_sockaddr_in", "addr %p, peername \"%s\"\n",
-                addr, inpeername ? inpeername : "[NIL]"));
-
     addr->sin_addr.s_addr = htonl(INADDR_ANY);
     addr->sin_family = AF_INET;
-    if (remote_port > 0) {
-        addr->sin_port = htons((u_short)remote_port);
-    } else if (netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
-				  NETSNMP_DS_LIB_DEFAULT_PORT) > 0) {
-        addr->sin_port = htons((u_short)netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
-						 NETSNMP_DS_LIB_DEFAULT_PORT));
-    } else {
-        addr->sin_port = htons(SNMP_PORT);
+    addr->sin_port = htons((u_short)SNMP_PORT);
+
+    {
+	int port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
+				      NETSNMP_DS_LIB_DEFAULT_PORT);
+
+	if (port != 0) {
+	    addr->sin_port = htons((u_short)port);
+	} else if (default_target != NULL)
+	    netsnmp_sockaddr_in2(addr, default_target, NULL);
     }
 
-    if (inpeername != NULL) {
+    if (inpeername != NULL && *inpeername != '\0') {
+	const char     *host, *port;
+	char           *peername = NULL;
+        char           *cp;
         /*
          * Duplicate the peername because we might want to mank around with
          * it.  
@@ -681,71 +812,147 @@ netsnmp_sockaddr_in(struct sockaddr_in *addr,
         cp = strchr(peername, ':');
         if (cp != NULL) {
             *cp = '\0';
-            cp++;
-            if (atoi(cp) != 0) {
-                DEBUGMSGTL(("netsnmp_sockaddr_in",
-                            "port number suffix :%d\n", atoi(cp)));
-                addr->sin_port = htons((u_short)atoi(cp));
+            port = cp + 1;
+            host = peername;
+        } else {
+            host = NULL;
+            port = peername;
+        }
+
+        /*
+         * Try to convert the user port specifier
+         */
+        if (port && *port == '\0')
+            port = NULL;
+
+        if (port != NULL) {
+            long int l;
+            char* ep;
+
+            DEBUGMSGTL(("netsnmp_sockaddr_in", "check user service %s\n",
+                        port));
+
+            l = strtol(port, &ep, 10);
+            if (ep != port && *ep == '\0' && 0 <= l && l <= 0x0ffff)
+                addr->sin_port = htons((u_short)l);
+            else {
+                if (host == NULL) {
+                    DEBUGMSGTL(("netsnmp_sockaddr_in",
+                                "servname not numeric, "
+				"check if it really is a destination)"));
+                    host = port;
+                    port = NULL;
+                } else {
+                    DEBUGMSGTL(("netsnmp_sockaddr_in",
+                                "servname not numeric"));
+                    free(peername);
+                    return 0;
+                }
             }
         }
 
-        for (cp = peername; *cp && isdigit((int) *cp); cp++);
-        if (!*cp && atoi(peername) != 0) {
-            /*
-             * Okay, it looks like just a port number.  
-             */
-            DEBUGMSGTL(("netsnmp_sockaddr_in", "totally numeric: %d\n",
-                        atoi(peername)));
-            addr->sin_port = htons((u_short)atoi(peername));
-        } else if (inet_addr(peername) != INADDR_NONE) {
-            /*
-             * It looks like an IP address.  
-             */
-            DEBUGMSGTL(("netsnmp_sockaddr_in", "IP address\n"));
-            addr->sin_addr.s_addr = inet_addr(peername);
-        } else {
-            /*
-             * Well, it must be a hostname then.  
-             */
-#ifdef  HAVE_GETHOSTBYNAME
-            struct hostent *hp = gethostbyname(peername);
+        /*
+         * Try to convert the user host specifier
+         */
+        if (host && *host == '\0')
+            host = NULL;
+
+        if (host != NULL) {
+            DEBUGMSGTL(("netsnmp_sockaddr_in",
+                        "check destination %s\n", host));
+
+#if HAVE_GETADDRINFO
+            memset(&hint, 0, sizeof hint);
+            hint.ai_flags = 0;
+            hint.ai_family = PF_INET;
+            hint.ai_socktype = SOCK_DGRAM;
+            hint.ai_protocol = 0;
+
+            err = getaddrinfo(peername, NULL, &hint, &addrs);
+            if (err != 0) {
+#if HAVE_GAI_STRERROR
+                snmp_log(LOG_ERR, "getaddrinfo: %s %s\n", peername,
+                         gai_strerror(err));
+#else
+                snmp_log(LOG_ERR, "getaddrinfo: %s (error %d)\n", peername,
+                         err);
+#endif
+                free(peername);
+                return 0;
+            }
+            if (addrs != NULL) {
+                DEBUGMSGTL(("netsnmp_sockaddr_in",
+                            "hostname (resolved okay)\n"));
+                memcpy(&addr->sin_addr,
+                       &((struct sockaddr_in *) addrs->ai_addr)->sin_addr,
+                       sizeof(struct in_addr));
+                freeaddrinfo(addrs);
+            }
+            else {
+                DEBUGMSGTL(("netsnmp_sockaddr_in",
+                            "Failed to resolve IPv4 hostname\n"));
+            }
+#elif HAVE_GETHOSTBYNAME
+            hp = gethostbyname(host);
             if (hp == NULL) {
                 DEBUGMSGTL(("netsnmp_sockaddr_in",
                             "hostname (couldn't resolve)\n"));
                 free(peername);
                 return 0;
+            } else if (hp->h_addrtype != AF_INET) {
+                DEBUGMSGTL(("netsnmp_sockaddr_in",
+                            "hostname (not AF_INET!)\n"));
+                free(peername);
+                return 0;
             } else {
-                if (hp->h_addrtype != AF_INET) {
-                    DEBUGMSGTL(("netsnmp_sockaddr_in",
-                                "hostname (not AF_INET!)\n"));
-                    free(peername);
-                    return 0;
-                } else {
-                    DEBUGMSGTL(("netsnmp_sockaddr_in",
-                                "hostname (resolved okay)\n"));
-                    memcpy(&(addr->sin_addr), hp->h_addr, hp->h_length);
-                }
+                DEBUGMSGTL(("netsnmp_sockaddr_in",
+                            "hostname (resolved okay)\n"));
+                memcpy(&addr->sin_addr, hp->h_addr, hp->h_length);
+            }
+#elif HAVE_GETIPNODEBYNAME
+            hp = getipnodebyname(peername, AF_INET, 0, &err);
+            if (hp == NULL) {
+                DEBUGMSGTL(("netsnmp_sockaddr_in",
+                            "hostname (couldn't resolve = %d)\n", err));
+                free(peername);
+                return 0;
             }
-#else                           /*HAVE_GETHOSTBYNAME */
             DEBUGMSGTL(("netsnmp_sockaddr_in",
-                        "hostname (no gethostbyname)\n"));
+                        "hostname (resolved okay)\n"));
+            memcpy(&(addr->sin_addr), hp->h_addr, hp->h_length);
+#else /* HAVE_GETIPNODEBYNAME */
+            /*
+             * There is no name resolving function available.
+             */
+            DEBUGMSGTL(("netsnmp_sockaddr_in",
+                        "no getaddrinfo()/getipnodebyname()/gethostbyname()\n"));
             free(peername);
             return 0;
-#endif                          /*HAVE_GETHOSTBYNAME */
+#endif /* HAVE_GETHOSTBYNAME */
         }
-    } else {
-        DEBUGMSGTL(("netsnmp_sockaddr_in", "NULL peername"));
-        return 0;
+	free(peername);
     }
+
+    /*
+     * Finished
+     */
+
     DEBUGMSGTL(("netsnmp_sockaddr_in", "return { AF_INET, %s:%hu }\n",
                 inet_ntoa(addr->sin_addr), ntohs(addr->sin_port)));
-    free(peername);
     return 1;
 }
 
 
+int
+netsnmp_sockaddr_in(struct sockaddr_in *addr,
+                    const char *inpeername, int remote_port)
+{
+    char buf[sizeof(int) * 3 + 2];
+    sprintf(buf, ":%u", remote_port);
+    return netsnmp_sockaddr_in2(addr, inpeername, remote_port ? buf : NULL);
+}
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 /*
  * The following functions provide the "com2sec" configuration token
  * functionality for compatibility.  
@@ -755,7 +962,7 @@ netsnmp_sockaddr_in(struct sockaddr_in *addr,
 #define EXAMPLE_COMMUNITY	"COMMUNITY"
 
 typedef struct _com2SecEntry {
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     unsigned long   network;
     unsigned long   mask;
     char            secName[VACMSTRINGLEN];
@@ -770,7 +977,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
 {
     char            secName[VACMSTRINGLEN];
     char            contextName[VACMSTRINGLEN];
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     char            source[SNMP_MAXBUF_SMALL];
     char           *cp = NULL;
     const char     *strmask = NULL;
@@ -818,7 +1025,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
             == 0) {
         config_perror("example config COMMUNITY not properly configured");
         return;
-    } else if (strlen(community) > (VACMSTRINGLEN - 1)) {
+    } else if (strlen(community) > (COMMUNITY_MAX_LEN - 1)) {
         config_perror("community name too long");
         return;
     }
@@ -969,7 +1176,7 @@ netsnmp_udp_com2SecList_free(void)
 void
 netsnmp_udp_agent_config_tokens_register(void)
 {
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     register_app_config_handler("com2sec", netsnmp_udp_parse_security,
                                 netsnmp_udp_com2SecList_free,
                                 "[-Cn CONTEXT] secName IPv4-network-address[/netmask] community");
@@ -985,7 +1192,7 @@ netsnmp_udp_agent_config_tokens_register(void)
  * parameters did not match any com2sec entry.  
  */
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 int
 netsnmp_udp_getSecName(void *opaque, int olength,
                        const char *community,
@@ -993,7 +1200,8 @@ netsnmp_udp_getSecName(void *opaque, int olength,
                        char **contextName)
 {
     com2SecEntry   *c;
-    struct sockaddr_in *from = (struct sockaddr_in *) opaque;
+    netsnmp_udp_addr_pair *addr_pair = (netsnmp_udp_addr_pair *) opaque;
+    struct sockaddr_in *from = (struct sockaddr_in *) &(addr_pair->remote_addr);
     char           *ztcommunity = NULL;
 
     if (secName != NULL) {
@@ -1015,7 +1223,7 @@ netsnmp_udp_getSecName(void *opaque, int olength,
      * name.  
      */
 
-    if (opaque == NULL || olength != sizeof(struct sockaddr_in) ||
+    if (opaque == NULL || olength != sizeof(netsnmp_udp_addr_pair) ||
         from->sin_family != AF_INET) {
         DEBUGMSGTL(("netsnmp_udp_getSecName",
 		    "no IPv4 source address in PDU?\n"));
@@ -1058,11 +1266,12 @@ netsnmp_udp_getSecName(void *opaque, int olength,
 
 
 netsnmp_transport *
-netsnmp_udp_create_tstring(const char *str, int local)
+netsnmp_udp_create_tstring(const char *str, int local,
+			   const char *default_target)
 {
     struct sockaddr_in addr;
 
-    if (netsnmp_sockaddr_in(&addr, str, 0)) {
+    if (netsnmp_sockaddr_in2(&addr, str, default_target)) {
         return netsnmp_udp_transport(&addr, local);
     } else {
         return NULL;
@@ -1091,10 +1300,10 @@ netsnmp_udp_ctor(void)
 {
     udpDomain.name = netsnmpUDPDomain;
     udpDomain.name_length = netsnmpUDPDomain_len;
-    udpDomain.prefix = calloc(2, sizeof(char *));
+    udpDomain.prefix = (const char**)calloc(2, sizeof(char *));
     udpDomain.prefix[0] = "udp";
 
-    udpDomain.f_create_from_tstring = netsnmp_udp_create_tstring;
+    udpDomain.f_create_from_tstring_new = netsnmp_udp_create_tstring;
     udpDomain.f_create_from_ostring = netsnmp_udp_create_ostring;
 
     netsnmp_tdomain_register(&udpDomain);
