@@ -68,12 +68,46 @@ static struct vacm_viewEntry *viewList = NULL, *viewScanPtr = NULL;
 static struct vacm_accessEntry *accessList = NULL, *accessScanPtr = NULL;
 static struct vacm_groupEntry *groupList = NULL, *groupScanPtr = NULL;
 
+/*
+ * Macro to extend view masks with 1 bits when shorter than subtree lengths
+ * REF: vacmViewTreeFamilyMask [RFC3415], snmpNotifyFilterMask [RFC3413]
+ */
+
+#define VIEW_MASK(viewPtr, idx, mask) \
+    ((idx >= viewPtr->viewMaskLen) ? mask : (viewPtr->viewMask[idx] & mask))
+
+/**
+ * Initilizes the VACM code.
+ * Specifically:
+ *  - adds a set of enums mapping view numbers to human readable names
+ */
+void
+init_vacm(void)
+{
+    /* views for access via get/set/send-notifications */
+    se_add_pair_to_slist(VACM_VIEW_ENUM_NAME, strdup("read"),
+                         VACM_VIEW_READ);
+    se_add_pair_to_slist(VACM_VIEW_ENUM_NAME, strdup("write"),
+                         VACM_VIEW_WRITE);
+    se_add_pair_to_slist(VACM_VIEW_ENUM_NAME, strdup("notify"),
+                         VACM_VIEW_NOTIFY);
+
+    /* views for permissions when receiving notifications */
+    se_add_pair_to_slist(VACM_VIEW_ENUM_NAME, strdup("log"),
+                         VACM_VIEW_LOG);
+    se_add_pair_to_slist(VACM_VIEW_ENUM_NAME, strdup("execute"),
+                         VACM_VIEW_EXECUTE);
+    se_add_pair_to_slist(VACM_VIEW_ENUM_NAME, strdup("net"),
+                         VACM_VIEW_NET);
+}
+
 void
 vacm_save(const char *token, const char *type)
 {
     struct vacm_viewEntry *vptr;
     struct vacm_accessEntry *aptr;
     struct vacm_groupEntry *gptr;
+    int i;
 
     for (vptr = viewList; vptr != NULL; vptr = vptr->next) {
         if (vptr->viewStorageType == ST_NONVOLATILE)
@@ -81,8 +115,18 @@ vacm_save(const char *token, const char *type)
     }
 
     for (aptr = accessList; aptr != NULL; aptr = aptr->next) {
-        if (aptr->storageType == ST_NONVOLATILE)
-            vacm_save_access(aptr, token, type);
+        if (aptr->storageType == ST_NONVOLATILE) {
+            /* Store the standard views (if set) */
+            if ( aptr->views[VACM_VIEW_READ  ][0] ||
+                 aptr->views[VACM_VIEW_WRITE ][0] ||
+                 aptr->views[VACM_VIEW_NOTIFY][0] )
+                vacm_save_access(aptr, token, type);
+            /* Store any other (valid) access views */
+            for ( i=VACM_VIEW_NOTIFY+1; i<VACM_MAX_VIEWS; i++ ) {
+                if ( aptr->views[i][0] )
+                    vacm_save_auth_access(aptr, token, type, i);
+            }
+        }
     }
 
     for (gptr = groupList; gptr != NULL; gptr = gptr->next) {
@@ -109,7 +153,7 @@ vacm_save_view(struct vacm_viewEntry *view, const char *token,
 
     cptr =
         read_config_save_octet_string(cptr, (u_char *) view->viewName + 1,
-                                      view->viewName[0] + 1);
+                                      view->viewName[0]);
     *cptr++ = ' ';
     cptr =
         read_config_save_objid(cptr, view->viewSubtree+1,
@@ -189,28 +233,62 @@ vacm_save_access(struct vacm_accessEntry *access_entry, const char *token,
                                       access_entry->contextPrefix[0] + 1);
 
     *cptr++ = ' ';
-    cptr = read_config_save_octet_string(cptr, (u_char *) access_entry->readView,
-                                         strlen(access_entry->readView) + 1);
+    cptr = read_config_save_octet_string(cptr, (u_char *) access_entry->views[VACM_VIEW_READ],
+                                         strlen(access_entry->views[VACM_VIEW_READ]) + 1);
     *cptr++ = ' ';
     cptr =
-        read_config_save_octet_string(cptr, (u_char *) access_entry->writeView,
-                                      strlen(access_entry->writeView) + 1);
+        read_config_save_octet_string(cptr, (u_char *) access_entry->views[VACM_VIEW_WRITE],
+                                      strlen(access_entry->views[VACM_VIEW_WRITE]) + 1);
     *cptr++ = ' ';
     cptr =
-        read_config_save_octet_string(cptr, (u_char *) access_entry->notifyView,
-                                      strlen(access_entry->notifyView) + 1);
+        read_config_save_octet_string(cptr, (u_char *) access_entry->views[VACM_VIEW_NOTIFY],
+                                      strlen(access_entry->views[VACM_VIEW_NOTIFY]) + 1);
 
     read_config_store(type, line);
 }
 
 void
-vacm_parse_config_access(const char *token, char *line)
+vacm_save_auth_access(struct vacm_accessEntry *access_entry,
+                      const char *token, const char *type, int authtype)
+{
+    char            line[4096];
+    char           *cptr;
+
+    memset(line, 0, sizeof(line));
+    snprintf(line, sizeof(line), "%s%s %d %d %d %d %d ",
+            token, "AuthAccess", access_entry->status,
+            access_entry->storageType, access_entry->securityModel,
+            access_entry->securityLevel, access_entry->contextMatch);
+    line[ sizeof(line)-1 ] = 0;
+    cptr = &line[strlen(line)]; /* the NULL */
+    cptr =
+        read_config_save_octet_string(cptr,
+                                      (u_char *) access_entry->groupName + 1,
+                                      access_entry->groupName[0] + 1);
+    *cptr++ = ' ';
+    cptr =
+        read_config_save_octet_string(cptr,
+                                      (u_char *) access_entry->contextPrefix + 1,
+                                      access_entry->contextPrefix[0] + 1);
+
+    snprintf(cptr, sizeof(line)-(cptr-line), " %d ", authtype);
+    while ( *cptr )
+        cptr++;
+
+    *cptr++ = ' ';
+    cptr = read_config_save_octet_string(cptr,
+                               (u_char *)access_entry->views[authtype],
+                                  strlen(access_entry->views[authtype]) + 1);
+
+    read_config_store(type, line);
+}
+
+char *
+_vacm_parse_config_access_common(struct vacm_accessEntry **aptr, char *line)
 {
     struct vacm_accessEntry access;
-    struct vacm_accessEntry *aptr;
-    char           *contextPrefix = (char *) &access.contextPrefix;
-    char           *groupName = (char *) &access.groupName;
-    char           *readView, *writeView, *notifyView;
+    char           *cPrefix = (char *) &access.contextPrefix;
+    char           *gName   = (char *) &access.groupName;
     size_t          len;
 
     access.status = atoi(line);
@@ -223,40 +301,77 @@ vacm_parse_config_access(const char *token, char *line)
     line = skip_token(line);
     access.contextMatch = atoi(line);
     line = skip_token(line);
-    len = sizeof(access.groupName);
-    line =
-        read_config_read_octet_string(line, (u_char **) & groupName, &len);
-    len = sizeof(access.contextPrefix);
-    line =
-        read_config_read_octet_string(line, (u_char **) & contextPrefix,
-                                      &len);
+    len  = sizeof(access.groupName);
+    line = read_config_read_octet_string(line, (u_char **) &gName,   &len);
+    len  = sizeof(access.contextPrefix);
+    line = read_config_read_octet_string(line, (u_char **) &cPrefix, &len);
 
-    aptr = vacm_createAccessEntry(access.groupName, access.contextPrefix,
+    *aptr = vacm_getAccessEntry(access.groupName,
+                                  access.contextPrefix,
                                   access.securityModel,
                                   access.securityLevel);
-    if (!aptr)
+    if (!*aptr)
+        *aptr = vacm_createAccessEntry(access.groupName,
+                                  access.contextPrefix,
+                                  access.securityModel,
+                                  access.securityLevel);
+    if (!*aptr)
+        return NULL;
+
+    (*aptr)->status = access.status;
+    (*aptr)->storageType   = access.storageType;
+    (*aptr)->securityModel = access.securityModel;
+    (*aptr)->securityLevel = access.securityLevel;
+    (*aptr)->contextMatch  = access.contextMatch;
+    return line;
+}
+
+void
+vacm_parse_config_access(const char *token, char *line)
+{
+    struct vacm_accessEntry *aptr;
+    char           *readView, *writeView, *notifyView;
+    size_t          len;
+
+    line = _vacm_parse_config_access_common(&aptr, line);
+    if (!line)
         return;
 
-    aptr->status = access.status;
-    aptr->storageType = access.storageType;
-    aptr->securityModel = access.securityModel;
-    aptr->securityLevel = access.securityLevel;
-    aptr->contextMatch = access.contextMatch;
-    readView = (char *) aptr->readView;
-    len = sizeof(aptr->readView);
+    readView = (char *) aptr->views[VACM_VIEW_READ];
+    len = sizeof(aptr->views[VACM_VIEW_READ]);
     line =
         read_config_read_octet_string(line, (u_char **) & readView, &len);
-    writeView = (char *) aptr->writeView;
-    len = sizeof(aptr->writeView);
+    writeView = (char *) aptr->views[VACM_VIEW_WRITE];
+    len = sizeof(aptr->views[VACM_VIEW_WRITE]);
     line =
         read_config_read_octet_string(line, (u_char **) & writeView, &len);
-    notifyView = (char *) aptr->notifyView;
-    len = sizeof(aptr->notifyView);
+    notifyView = (char *) aptr->views[VACM_VIEW_NOTIFY];
+    len = sizeof(aptr->views[VACM_VIEW_NOTIFY]);
     line =
         read_config_read_octet_string(line, (u_char **) & notifyView,
                                       &len);
 }
 
+void
+vacm_parse_config_auth_access(const char *token, char *line)
+{
+    struct vacm_accessEntry *aptr;
+    int             authtype;
+    char           *view;
+    size_t          len;
+
+    line = _vacm_parse_config_access_common(&aptr, line);
+    if (!line)
+        return;
+
+    authtype = atoi(line);
+    line = skip_token(line);
+
+    view = (char *) aptr->views[authtype];
+    len  = sizeof(aptr->views[authtype]);
+    line = read_config_read_octet_string(line, (u_char **) & view, &len);
+}
+
 /*
  * vacm_save_group(): saves a group entry to the persistent cache 
  */
@@ -318,7 +433,7 @@ vacm_parse_config_group(const char *token, char *line)
 }
 
 struct vacm_viewEntry *
-vacm_getViewEntry(const char *viewName,
+netsnmp_view_get(struct vacm_viewEntry *head, const char *viewName,
                   oid * viewSubtree, size_t viewSubtreeLen, int mode)
 {
     struct vacm_viewEntry *vp, *vpret = NULL;
@@ -331,29 +446,28 @@ vacm_getViewEntry(const char *viewName,
         return NULL;
     view[0] = glen;
     strcpy(view + 1, viewName);
-    for (vp = viewList; vp; vp = vp->next) {
+    for (vp = head; vp; vp = vp->next) {
         if (!memcmp(view, vp->viewName, glen + 1)
             && viewSubtreeLen >= (vp->viewSubtreeLen - 1)) {
             int             mask = 0x80, maskpos = 0;
             int             oidpos;
             found = 1;
 
-            if (mode != VACM_MODE_IGNORE_MASK) {  /* check the mask */
-                for (oidpos = 0;
-                     found && oidpos < (int) vp->viewSubtreeLen - 1;
-                     oidpos++) {
-                    if ((vp->viewMask[maskpos] & mask) != 0) {
-                        if (viewSubtree[oidpos] !=
-                            vp->viewSubtree[oidpos + 1])
-                            found = 0;
-                    }
-                    if (mask == 1) {
-                        mask = 0x80;
-                        maskpos++;
-                    } else
-                        mask >>= 1;
+            for (oidpos = 0;
+                 found && oidpos < (int) vp->viewSubtreeLen - 1;
+                 oidpos++) {
+                if (mode==VACM_MODE_IGNORE_MASK || (VIEW_MASK(vp, maskpos, mask)) != 0) {
+                    if (viewSubtree[oidpos] !=
+                        vp->viewSubtree[oidpos + 1])
+                        found = 0;
                 }
+                if (mask == 1) {
+                    mask = 0x80;
+                    maskpos++;
+                } else
+                    mask >>= 1;
             }
+
             if (found) {
                 /*
                  * match successful, keep this node if its longer than
@@ -403,8 +517,8 @@ vacm_getViewEntry(const char *viewName,
  *                         portions.
  */
 int
-vacm_checkSubtree(const char *viewName,
-                  oid * viewSubtree, size_t viewSubtreeLen)
+netsnmp_view_subtree_check(struct vacm_viewEntry *head, const char *viewName,
+                           oid * viewSubtree, size_t viewSubtreeLen)
 {
     struct vacm_viewEntry *vp, *vpShorter = NULL, *vpLonger = NULL;
     char            view[VACMSTRINGLEN];
@@ -415,7 +529,8 @@ vacm_checkSubtree(const char *viewName,
         return VACM_NOTINVIEW;
     view[0] = glen;
     strcpy(view + 1, viewName);
-    for (vp = viewList; vp; vp = vp->next) {
+    DEBUGMSGTL(("9:vacm:checkSubtree", "view %s\n", viewName));
+    for (vp = head; vp; vp = vp->next) {
         if (!memcmp(view, vp->viewName, glen + 1)) {
             /*
              * If the subtree defined in the view is shorter than or equal
@@ -433,7 +548,7 @@ vacm_checkSubtree(const char *viewName,
                 for (oidpos = 0;
                      found && oidpos < (int) vp->viewSubtreeLen - 1;
                      oidpos++) {
-                    if ((vp->viewMask[maskpos] & mask) != 0) {
+                    if (VIEW_MASK(vp, maskpos, mask) != 0) {
                         if (viewSubtree[oidpos] !=
                             vp->viewSubtree[oidpos + 1])
                             found = 0;
@@ -451,6 +566,7 @@ vacm_checkSubtree(const char *viewName,
                      * the previous or (equal and lexicographically greater
                      * than the previous). 
                      */
+                    DEBUGMSGTL(("9:vacm:checkSubtree", " %s matched?\n", vp->viewName));
     
                     if (vpShorter == NULL
                         || vp->viewSubtreeLen > vpShorter->viewSubtreeLen
@@ -480,7 +596,7 @@ vacm_checkSubtree(const char *viewName,
                 for (oidpos = 0;
                      found && oidpos < (int) viewSubtreeLen;
                      oidpos++) {
-                    if ((vp->viewMask[maskpos] & mask) != 0) {
+                    if (VIEW_MASK(vp, maskpos, mask) != 0) {
                         if (viewSubtree[oidpos] !=
                             vp->viewSubtree[oidpos + 1])
                             found = 0;
@@ -498,6 +614,7 @@ vacm_checkSubtree(const char *viewName,
                      * with a different view type, then parts of the subtree 
                      * are included and others are excluded, so return UNKNOWN.
                      */
+                    DEBUGMSGTL(("9:vacm:checkSubtree", " %s matched?\n", vp->viewName));
                     if (vpLonger != NULL
                         && (vpLonger->viewType != vp->viewType)) {
                         DEBUGMSGTL(("vacm:checkSubtree", ", %s\n", "unknown"));
@@ -510,6 +627,7 @@ vacm_checkSubtree(const char *viewName,
             }
         }
     }
+    DEBUGMSGTL(("9:vacm:checkSubtree", " %s matched\n", vp->viewName));
 
     /*
      * If we found a matching view subtree with a longer OID than the provided
@@ -556,7 +674,7 @@ vacm_scanViewNext(void)
 }
 
 struct vacm_viewEntry *
-vacm_createViewEntry(const char *viewName,
+netsnmp_view_create(struct vacm_viewEntry **head, const char *viewName,
                      oid * viewSubtree, size_t viewSubtreeLen)
 {
     struct vacm_viewEntry *vp, *lp, *op = NULL;
@@ -582,7 +700,7 @@ vacm_createViewEntry(const char *viewName,
     memcpy(vp->viewSubtree + 1, viewSubtree, viewSubtreeLen * sizeof(oid));
     vp->viewSubtreeLen = viewSubtreeLen + 1;
 
-    lp = viewList;
+    lp = *head;
     while (lp) {
         cmp = memcmp(lp->viewName, vp->viewName, glen + 1);
         cmp2 = snmp_oid_compare(lp->viewSubtree, lp->viewSubtreeLen,
@@ -598,24 +716,24 @@ vacm_createViewEntry(const char *viewName,
     if (op)
         op->next = vp;
     else
-        viewList = vp;
+        *head = vp;
     return vp;
 }
 
 void
-vacm_destroyViewEntry(const char *viewName,
+netsnmp_view_destroy(struct vacm_viewEntry **head, const char *viewName,
                       oid * viewSubtree, size_t viewSubtreeLen)
 {
     struct vacm_viewEntry *vp, *lastvp = NULL;
 
-    if (viewList && !strcmp(viewList->viewName + 1, viewName)
-        && viewList->viewSubtreeLen == viewSubtreeLen
-        && !memcmp((char *) viewList->viewSubtree, (char *) viewSubtree,
+    if ((*head) && !strcmp((*head)->viewName + 1, viewName)
+        && (*head)->viewSubtreeLen == viewSubtreeLen
+        && !memcmp((char *) (*head)->viewSubtree, (char *) viewSubtree,
                    viewSubtreeLen * sizeof(oid))) {
-        vp = viewList;
-        viewList = viewList->next;
+        vp = (*head);
+        (*head) = (*head)->next;
     } else {
-        for (vp = viewList; vp; vp = vp->next) {
+        for (vp = (*head); vp; vp = vp->next) {
             if (!strcmp(vp->viewName + 1, viewName)
                 && vp->viewSubtreeLen == viewSubtreeLen
                 && !memcmp((char *) vp->viewSubtree, (char *) viewSubtree,
@@ -634,11 +752,11 @@ vacm_destroyViewEntry(const char *viewName,
 }
 
 void
-vacm_destroyAllViewEntries(void)
+netsnmp_view_clear(struct vacm_viewEntry **head)
 {
     struct vacm_viewEntry *vp;
-    while ((vp = viewList)) {
-        viewList = vp->next;
+    while ((vp = (*head))) {
+        (*head) = vp->next;
         if (vp->reserved)
             free(vp->reserved);
         free(vp);
@@ -869,11 +987,11 @@ vacm_createAccessEntry(const char *groupName,
             break;
         if (cmp < 0)
             goto next;
-        if (lp->securityModel < securityModel)
-            break;
         if (lp->securityModel > securityModel)
+            break;
+        if (lp->securityModel < securityModel)
             goto next;
-        if (lp->securityLevel < securityLevel)
+        if (lp->securityLevel > securityLevel)
             break;
       next:
         op = lp;
@@ -951,15 +1069,56 @@ store_vacm(int majorID, int minorID, void *serverarg, void *clientarg)
 }
 
 /*
- * returns 1 if vacm has *any* configuration entries in it (regardless
- * of weather or not there is enough to make a decision based on it),
+ * returns 1 if vacm has *any* (non-built-in) configuration entries,
+ * regardless of whether or not there is enough to make a decision,
  * else return 0 
  */
 int
 vacm_is_configured(void)
 {
-    if (viewList == NULL && accessList == NULL && groupList == NULL) {
+    if (accessList == NULL && groupList == NULL) {
         return 0;
     }
     return 1;
 }
+
+/*
+ * backwards compatability
+ */
+struct vacm_viewEntry *
+vacm_getViewEntry(const char *viewName,
+                  oid * viewSubtree, size_t viewSubtreeLen, int mode)
+{
+    return netsnmp_view_get( viewList, viewName, viewSubtree, viewSubtreeLen,
+                             mode);
+}
+
+int
+vacm_checkSubtree(const char *viewName,
+                  oid * viewSubtree, size_t viewSubtreeLen)
+{
+    return netsnmp_view_subtree_check( viewList, viewName, viewSubtree,
+                                       viewSubtreeLen);
+}
+
+struct vacm_viewEntry *
+vacm_createViewEntry(const char *viewName,
+                     oid * viewSubtree, size_t viewSubtreeLen)
+{
+    return netsnmp_view_create( &viewList, viewName, viewSubtree,
+                                viewSubtreeLen);
+}
+
+void
+vacm_destroyViewEntry(const char *viewName,
+                      oid * viewSubtree, size_t viewSubtreeLen)
+{
+    netsnmp_view_destroy( &viewList, viewName, viewSubtree, viewSubtreeLen);
+}
+
+void
+vacm_destroyAllViewEntries(void)
+{
+    netsnmp_view_clear( &viewList );
+}
+
